/**
 * WooCommerce Integration for Minimal Chatbot - Add-on JavaScript
 * Version: 1.0.0
 *
 * Handles displaying WooCommerce product cards within the chat widget
 * based on data received in AJAX responses.
 */
jQuery(document).ready(function($) {

    // --- Helper Functions ---

    /**
     * Simple HTML escaping function for data used in attributes or text nodes.
     * @param {string} text The text to escape.
     * @returns {string} The escaped text.
     */
    function escapeHtml(text) {
        if (text === null || typeof text === 'undefined') {
            return '';
        }
        const map = {
            '&': '&amp;',
            '<': '&lt;',
            '>': '&gt;',
            '"': '&quot;',
            "'": '&#039;'
        };
        return String(text).replace(/[&<>"']/g, function(m) { return map[m]; });
    }

     /**
     * Creates the HTML for a single product card.
     * @param {object} product The product data object from the AJAX response.
     * @returns {jQuery|null} The jQuery object for the product card link or null if data is invalid.
     */
    function createProductCardHtml(product) {
        // Validate required product data
        if (!product || typeof product !== 'object' || !product.id || !product.title || !product.url || !product.price_html) {
            console.warn("MC WC Addon: Invalid product data received for card generation.", product);
            return null;
        }

        // Prepare data elements
        const imageUrl = product.image_url || ''; // Use empty string if no image
        const imageAlt = product.title || 'Produktbild'; // Use title as alt text
        const productTitle = product.title;
        const productUrl = product.url;
        const priceHtml = product.price_html; // Use the HTML provided by WooCommerce
        const stockStatus = product.stock_status || 'outofstock'; // Default to outofstock if missing
        const stockQuantity = product.stock_quantity; // Might be null
        const shortDescription = product.short_description || ''; // Might be empty

        // Determine stock status text and class
        let stockText = 'N/A';
        let stockClass = 'stock-status-outofstock'; // Default class

        switch (stockStatus) {
            case 'instock':
                stockText = 'Auf Lager'; // Translate if needed: __('In Stock', 'wc-minichat-addon');
                stockClass = 'stock-status-instock';
                if (stockQuantity !== null && stockQuantity > 0) {
                    // Translate if needed: sprintf(__('(%d available)', 'wc-minichat-addon'), stockQuantity);
                    stockText += ' (' + parseInt(stockQuantity, 10) + ' verfügbar)';
                }
                break;
            case 'outofstock':
                stockText = 'Nicht vorrätig'; // Translate if needed: __('Out of Stock', 'wc-minichat-addon');
                stockClass = 'stock-status-outofstock';
                break;
            case 'onbackorder':
                stockText = 'Nachbestellbar'; // Translate if needed: __('On Backorder', 'wc-minichat-addon');
                stockClass = 'stock-status-onbackorder';
                break;
        }

        // Build the card HTML using jQuery (more readable than string concatenation)
        const $cardLink = $('<a>')
            .addClass('minichat-product-card')
            .attr('href', escapeHtml(productUrl)) // Ensure URL is escaped for attribute
            .attr('target', '_blank') // Open product link in a new tab
            .attr('rel', 'noopener noreferrer'); // Security for target="_blank"

        const $image = $('<img>')
            .addClass('minichat-product-image')
            .attr('src', escapeHtml(imageUrl)) // Ensure URL is escaped
            .attr('alt', escapeHtml(imageAlt)); // Escape alt text

        const $details = $('<div>').addClass('minichat-product-details');

        const $title = $('<div>')
            .addClass('minichat-product-title')
            .text(productTitle); // Use text() for title to prevent XSS

        const $price = $('<div>')
            .addClass('minichat-product-price')
            .html(priceHtml); // Use html() to render WC price format (e.g., <del>, <ins>)

        const $stock = $('<div>')
            .addClass('minichat-product-stock')
            .addClass(stockClass) // Add status-specific class
            .text(stockText); // Use text() for stock status

        // Append details
        $details.append($title, $price, $stock);

        // Append image and details to the card link
        $cardLink.append($image, $details);

        // Optional: Add short description if present
        if (shortDescription) {
            const $desc = $('<p>')
                .addClass('minichat-product-short-desc')
                .css({'font-size': '0.8em', 'margin-top': '5px', 'color': '#666'}) // Basic inline style for demo
                .text(shortDescription); // Use text() for description
            $details.append($desc); // Append description within details container
        }

        return $cardLink;
    }

    // --- Event Listener for Core Chatbot Messages ---

    // Listen for the custom event triggered by minimal-chatbot.js after a message is displayed
    $(document).on('minichat:after_display_message', function(event, eventData) {

        // Basic check if event data exists
        if (!eventData || typeof eventData !== 'object') {
            return;
        }

        // Only process messages from the 'bot'
        if (eventData.sender !== 'bot') {
            return;
        }

        // Check if the AJAX response data and product data are present
        if (!eventData.ajaxResponseData || !Array.isArray(eventData.ajaxResponseData.minichat_wc_products) || eventData.ajaxResponseData.minichat_wc_products.length === 0) {
            // No product data found in this response, do nothing
            return;
        }

        // Get the target message element (the <div class="message-wrapper...">)
        const $messageWrapper = eventData.messageElement;
        if (!$messageWrapper || $messageWrapper.length === 0) {
            console.error("MC WC Addon: Could not find target message element in event data.");
            return;
        }

        // Find the actual message bubble within the wrapper to append the list to
        const $messageBubble = $messageWrapper.find('.message.bot-message');
        if ($messageBubble.length === 0) {
            console.error("MC WC Addon: Could not find message bubble element.");
            return;
        }

        // Get the product data array
        const products = eventData.ajaxResponseData.minichat_wc_products;

        console.log("MC WC Addon: Received product data, attempting to render cards.", products); // Debug log

        // Create a container for the product cards if it doesn't exist yet for this message
        let $productList = $messageBubble.find('.minichat-product-list');
        if ($productList.length === 0) {
            $productList = $('<div>').addClass('minichat-product-list');
            $messageBubble.append($productList); // Append list inside the message bubble
        }

        // Iterate through the product data and create/append cards
        let cardsAdded = 0;
        products.forEach(function(product) {
            const $cardHtml = createProductCardHtml(product);
            if ($cardHtml) {
                $productList.append($cardHtml);
                cardsAdded++;
            }
        });

        // If cards were added, maybe force scroll again (optional, core JS might handle it)
        if (cardsAdded > 0) {
            // Find the main messages container to trigger scroll (assuming core JS structure)
            const $messagesContainer = $('#minichat-messages');
            if ($messagesContainer.length > 0) {
                // Use a short delay to ensure DOM update is complete before scrolling
                setTimeout(function() {
                    $messagesContainer.stop().animate({
                        scrollTop: $messagesContainer[0].scrollHeight
                    }, 300);
                }, 100);
            }
            console.log("MC WC Addon: Rendered " + cardsAdded + " product cards.");
        }
    });

}); // End jQuery ready