<?php
// includes/admin-page-feedback.php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// --- WP_List_Table for displaying feedback ---
if (!class_exists('WP_List_Table')) {
    require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}

class Minichat_Feedback_List_Table extends WP_List_Table {

    function get_columns() {
        $columns = [
            //'cb'            => '<input type="checkbox" />', // Optional: Bulk actions checkbox
            'timestamp'     => __('Date', 'minimal-chatbot'),
            'content'       => __('Bot Response', 'minimal-chatbot'),
            'feedback_rating' => __('Rating', 'minimal-chatbot'),
            'feedback_text' => __('Comment', 'minimal-chatbot'),
            'user_id'       => __('User ID', 'minimal-chatbot'), // Show anonymized User ID
        ];
        return $columns;
    }

    function prepare_items() {
        global $wpdb;
        $table_name = $wpdb->prefix . 'minichat_history';

        $columns = $this->get_columns();
        $hidden = []; // Columns to hide
        $sortable = $this->get_sortable_columns();
        $this->_column_headers = [$columns, $hidden, $sortable];

        $per_page = 20;
        $current_page = $this->get_pagenum();
        $offset = ($current_page - 1) * $per_page;

        // Base query to get feedback entries
        $query = "SELECT id, timestamp, user_id, content, feedback_rating, feedback_text
                  FROM {$table_name}
                  WHERE feedback_rating IS NOT NULL AND role = 'assistant'"; // Only show bot messages with feedback

        // Ordering
        $orderby = isset($_GET['orderby']) ? sanitize_key($_GET['orderby']) : 'timestamp';
        $order = isset($_GET['order']) ? strtoupper(sanitize_key($_GET['order'])) : 'DESC';
        if (!array_key_exists($orderby, $this->get_sortable_columns())) {
            $orderby = 'timestamp';
        }
        if (!in_array($order, ['ASC', 'DESC'])) {
            $order = 'DESC';
        }
        $query .= $wpdb->prepare(" ORDER BY %s %s", $orderby, $order);


        // Total items
        $total_items_query = str_replace('SELECT id, timestamp, user_id, content, feedback_rating, feedback_text', 'SELECT COUNT(id)', $query);
         // Remove ORDER BY and LIMIT for count query - Find last ORDER BY and remove it
         $orderby_pos = strripos($total_items_query, 'ORDER BY');
         if ($orderby_pos !== false) {
             $total_items_query = substr($total_items_query, 0, $orderby_pos);
         }
        $total_items = $wpdb->get_var($total_items_query);


        // Pagination
        $query .= $wpdb->prepare(" LIMIT %d OFFSET %d", $per_page, $offset);

        // Fetch data
        $this->items = $wpdb->get_results($query, ARRAY_A);

        // Set pagination arguments
        $this->set_pagination_args([
            'total_items' => $total_items,
            'per_page'    => $per_page,
            'total_pages' => ceil($total_items / $per_page)
        ]);
    }

    function get_sortable_columns() {
        $sortable_columns = [
            'timestamp'     => ['timestamp', false], // True means it's already sorted by default
            'feedback_rating' => ['feedback_rating', false],
        ];
        return $sortable_columns;
    }

    // How to display columns
    function column_default($item, $column_name) {
        switch ($column_name) {
            case 'timestamp':
                return esc_html(wp_date(get_option('date_format') . ' ' . get_option('time_format'), strtotime($item['timestamp'])));
            case 'content':
                // Truncate long responses
                return esc_html(wp_html_excerpt($item['content'], 150, '...'));
            case 'feedback_rating':
                if ($item['feedback_rating'] == 1) {
                    return '<span style="color:green;"><span class="dashicons dashicons-thumbs-up"></span> Positive</span>';
                } elseif ($item['feedback_rating'] == 0) {
                     return '<span style="color:red;"><span class="dashicons dashicons-thumbs-down"></span> Negative</span>';
                }
                return 'N/A';
             case 'feedback_text':
                 return !empty($item['feedback_text']) ? nl2br(esc_html($item['feedback_text'])) : '<em>(No comment)</em>';
            case 'user_id':
                // Display only part of the user ID for privacy/brevity if needed
                return '...' . esc_html(substr($item['user_id'], -8));
            default:
                return print_r($item, true); // Show the whole array for other columns (debug)
        }
    }

     // Add hover actions (optional)
     function column_content($item) {
         $actions = [
             //'view_full' => sprintf('<a href="#">View Full</a>'), // Example action
             //'delete'    => sprintf('<a href="?page=%s&action=delete_feedback&id=%s&_wpnonce=%s">Delete</a>', $_REQUEST['page'], $item['id'], wp_create_nonce('minichat_delete_feedback')),
         ];
         return sprintf('%1$s %2$s',
             esc_html(wp_html_excerpt($item['content'], 150, '...')),
             $this->row_actions($actions)
         );
     }


    // For checkboxes (if using bulk actions)
    /*
    function column_cb($item) {
        return sprintf(
            '<input type="checkbox" name="feedback_ids[]" value="%s" />', $item['id']
        );
    }
    */

    function no_items() {
        _e('No feedback entries found.', 'minimal-chatbot');
    }
}


/**
 * Renders the HTML for the Feedback admin page.
 */
function minichat_feedback_page_html() {
    if (!current_user_can('manage_options')) {
        return;
    }

    // Create an instance of our package class...
    $feedback_list_table = new Minichat_Feedback_List_Table();
    // Fetch, prepare, sort, and filter our data...
    $feedback_list_table->prepare_items();

    ?>
    <div class="wrap">
        <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
        <p>Here you can view the feedback submitted by users for the chatbot responses.</p>

        <!-- Forms are required for bulk actions -->
        <form id="feedback-filter" method="get">
            <!-- For plugins, we also need to ensure that the form posts back to our current page -->
            <input type="hidden" name="page" value="<?php echo esc_attr($_REQUEST['page']); ?>" />
            <!-- Now we can render the completed list table -->
            <?php $feedback_list_table->display(); ?>
        </form>
    </div>
    <?php
}