<?php
// includes/admin-page-status.php

if (!defined('ABSPATH')) { exit; }

// Include the Users List Table class only if it wasn't already included (safety check)
// Assuming MINICHAT_PLUGIN_DIR is defined correctly in the main plugin file
if (!class_exists('Minichat_Users_List_Table') && defined('MINICHAT_PLUGIN_DIR') && file_exists(MINICHAT_PLUGIN_DIR . 'includes/class-minichat-users-list-table.php')) {
    require_once MINICHAT_PLUGIN_DIR . 'includes/class-minichat-users-list-table.php';
} elseif (!class_exists('Minichat_Users_List_Table')) {
    // Fallback or error handling if the class cannot be included
    // This prevents a fatal error if the class is missing but allows the page to potentially load partially
    if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) {
        error_log("MC[Status Page]: WARN - Minichat_Users_List_Table class not found or MINICHAT_PLUGIN_DIR not defined.");
    }
    // We don't exit here, but the table won't render later.
}


/**
 * Calculates and returns basic statistics.
 * @return array Array of statistics.
 */
function minichat_get_basic_stats() {
    global $wpdb;
    // Ensure table name prefix is handled correctly
    $table_name = $wpdb->prefix . 'minichat_history';
    $stats = [ // Initialize with defaults
        'total_messages' => 0,
        'unique_users' => 0,
        'feedback_total' => 0,
        'feedback_positive' => 0,
        'feedback_negative' => 0,
        'feedback_positive_percent' => 0,
        'feedback_negative_percent' => 0,
        'avg_messages_per_user' => 0,
        'error' => null // Initialize error state as null
    ];

    // Check if table exists using WPDB method
    if ($wpdb->get_var($wpdb->prepare("SHOW TABLES LIKE %s", $table_name)) !== $table_name) {
        $stats['error'] = __('Error: Chat history table not found.', 'minimal-chatbot') . ' (' . esc_html($table_name) . ')';
        if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) {
             error_log("MC[Stats]: DB Error - Table '{$table_name}' not found.");
        }
        return $stats;
    }

    // Get statistics using $wpdb->prepare for all queries where variables are involved, even if static here
    // Total messages
    $stats['total_messages'] = (int) $wpdb->get_var( "SELECT COUNT(*) FROM {$table_name}" );

    // Unique users
    $stats['unique_users'] = (int) $wpdb->get_var( "SELECT COUNT(DISTINCT user_id) FROM {$table_name}" );

    // Feedback stats - Ensure role='assistant' is part of the condition
    $stats['feedback_total'] = (int) $wpdb->get_var($wpdb->prepare(
        "SELECT COUNT(*) FROM {$table_name} WHERE feedback_rating IS NOT NULL AND role = %s",
        'assistant'
    ));
    $stats['feedback_positive'] = (int) $wpdb->get_var($wpdb->prepare(
        "SELECT COUNT(*) FROM {$table_name} WHERE feedback_rating = %d AND role = %s",
        1, 'assistant'
    ));
    $stats['feedback_negative'] = (int) $wpdb->get_var($wpdb->prepare(
        "SELECT COUNT(*) FROM {$table_name} WHERE feedback_rating = %d AND role = %s",
        0, 'assistant'
    ));

    // Check for DB errors after queries
    if ($wpdb->last_error) {
        $stats['error'] = __('Database error during statistics calculation.', 'minimal-chatbot');
         if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) {
             error_log("MC[Stats]: DB Error - " . $wpdb->last_error);
        }
        // Return potentially partial stats along with the error message
        return $stats;
    }

    // Calculate percentages safely
    $stats['feedback_positive_percent'] = $stats['feedback_total'] > 0 ? round(($stats['feedback_positive'] / $stats['feedback_total']) * 100, 1) : 0;
    $stats['feedback_negative_percent'] = $stats['feedback_total'] > 0 ? round(($stats['feedback_negative'] / $stats['feedback_total']) * 100, 1) : 0;

    // Avg messages per user safely
    $stats['avg_messages_per_user'] = $stats['unique_users'] > 0 ? round($stats['total_messages'] / $stats['unique_users'], 1) : 0;

    return $stats;
}


/**
 * Renders the HTML for the Status & Statistics admin page.
 */
function minichat_status_page_html() {
    // Check user capability first
    if (!current_user_can('manage_options')) {
        wp_die(__('You do not have sufficient permissions to access this page.'));
        return;
    }

    // Get stats
    $stats = minichat_get_basic_stats();

    // Create and prepare the users list table
    $users_list_table = null; // Initialize
    // Check if the class exists before instantiation
    if (class_exists('Minichat_Users_List_Table')) {
        $users_list_table = new Minichat_Users_List_Table();
        // Call prepare_items only if the object was created successfully
        if ($users_list_table instanceof Minichat_Users_List_Table) {
             $users_list_table->prepare_items();
        } else {
            // This case should ideally not happen if class_exists is true, but as a safeguard:
            $users_list_table = null; // Ensure it's null if instantiation failed
            echo '<div class="notice notice-error"><p>' . __('Error: Could not create the user list table object.', 'minimal-chatbot') . '</p></div>';
        }
    } else {
         echo '<div class="notice notice-warning"><p>' . __('User list table class not found. Please ensure all plugin files are correctly installed.', 'minimal-chatbot') . '</p></div>';
    }

    ?>
    <div class="wrap">
        <h1><?php echo esc_html(get_admin_page_title()); ?></h1>

        <?php // Display database error prominently if it occurred during stats fetching ?>
        <?php if (isset($stats['error']) && $stats['error'] !== null): ?>
            <div class="notice notice-error"><p><strong><?php _e('Statistics Error:', 'minimal-chatbot'); ?></strong> <?php echo esc_html($stats['error']); ?></p></div>
        <?php endif; ?>

        <?php // Display statistics section only if no critical error stopped fetching (e.g., table not found) ?>
        <?php if (!isset($stats['error']) || strpos($stats['error'], 'table not found') === false): ?>
            <h2><?php _e('Usage Statistics', 'minimal-chatbot'); ?></h2>
            <div id="dashboard-widgets-wrap">
                <div id="dashboard-widgets" class="metabox-holder">
                    <div id="postbox-container-1" class="postbox-container">
                        <div class="meta-box-sortables">
                            <div class="postbox">
                                <h2 class="hndle"><span><?php _e('Overall Usage', 'minimal-chatbot'); ?></span></h2>
                                <div class="inside">
                                    <ul>
                                        <li><strong><?php _e('Total Messages:', 'minimal-chatbot'); ?></strong> <?php echo esc_html($stats['total_messages']); ?></li>
                                        <li><strong><?php _e('Unique Users (approx.):', 'minimal-chatbot'); ?></strong> <?php echo esc_html($stats['unique_users']); ?></li>
                                        <li><strong><?php _e('Avg. Messages per User:', 'minimal-chatbot'); ?></strong> <?php echo esc_html($stats['avg_messages_per_user']); ?></li>
                                    </ul>
                                </div>
                            </div>
                            <div class="postbox">
                                <h2 class="hndle"><span><?php _e('Feedback Summary', 'minimal-chatbot'); ?></span></h2>
                                <div class="inside">
                                    <ul>
                                        <li><strong><?php _e('Total Feedback Entries:', 'minimal-chatbot'); ?></strong> <?php echo esc_html($stats['feedback_total']); ?></li>
                                        <li><strong><?php _e('Positive Feedback:', 'minimal-chatbot'); ?></strong> <?php echo esc_html($stats['feedback_positive']); ?> (<?php echo esc_html($stats['feedback_positive_percent']); ?>%)</li>
                                        <li><strong><?php _e('Negative Feedback:', 'minimal-chatbot'); ?></strong> <?php echo esc_html($stats['feedback_negative']); ?> (<?php echo esc_html($stats['feedback_negative_percent']); ?>%)</li>
                                    </ul>
                                </div>
                            </div>
                            <?php // Add more stat boxes here if needed ?>
                        </div>
                    </div>
                    <div id="postbox-container-2" class="postbox-container">
                        <div class="meta-box-sortables">
                            <?php // Placeholder for potential future content in the second column ?>
                        </div>
                    </div>
                </div>
                <div class="clear"></div>
            </div>
        <?php endif; ?>


        <hr>
        <h2><?php _e('User Activity Overview', 'minimal-chatbot'); ?></h2>
        <?php // Only display the table section if the table object was successfully created ?>
        <?php if ($users_list_table): ?>
            <p><?php _e('Showing users based on chat history entries.', 'minimal-chatbot'); ?></p>
            <form id="users-filter" method="get">
                <?php // The page parameter is usually handled by WP_List_Table automatically when generating links ?>
                <input type="hidden" name="page" value="<?php echo isset($_REQUEST['page']) ? esc_attr($_REQUEST['page']) : 'minimal-chatbot-status'; ?>" />
                <?php
                // Display the table content
                $users_list_table->display();
                ?>
            </form>
        <?php else: ?>
             <p><?php _e('User activity table could not be loaded.', 'minimal-chatbot'); ?></p>
        <?php endif; ?>

    </div>
    <style> /* Basic styling for two columns */ #dashboard-widgets.metabox-holder { display: flex; flex-wrap: wrap; gap: 20px; } #postbox-container-1 { flex: 1 1 300px; /* Allow flex grow/shrink, base width 300px */ min-width: 250px; } #postbox-container-2 { flex: 2 1 400px; /* Allow flex grow/shrink, base width 400px */ } .postbox ul { margin: 0; padding: 0 0 0 20px; list-style: disc; } .postbox ul li { margin-bottom: 8px; } /* Ensure clear works */ .clear { clear: both; } </style>
    <?php
} // <-- Dies ist die korrigierte schließende Klammer

?>