<?php
/**
 * Admin Page for Minimal Chatbot - WooCommerce Integration Settings
 *
 * Creates the admin page under WooCommerce menu, displays status,
 * and handles settings registration for the WC integration.
 * NEU: Zeigt Anzahl gefundener veröffentlichter Produkte an.
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Define constants used within this file (consistent with add-on main file)
if (!defined('MINICHAT_WC_ADDON_ENABLE_OPTION')) {
    define('MINICHAT_WC_ADDON_ENABLE_OPTION', 'minichat_wc_integration_enabled');
}
if (!defined('MINICHAT_CORE_PLUGIN_SLUG')) {
    define('MINICHAT_CORE_PLUGIN_SLUG', 'minimal-chatbot/minimal-chatbot.php');
}
if (!defined('WOOCOMMERCE_PLUGIN_SLUG')) {
    define('WOOCOMMERCE_PLUGIN_SLUG', 'woocommerce/woocommerce.php');
}


// Hook to register settings, sections, and fields for this specific admin page
add_action('admin_init', 'minichat_wc_addon_register_admin_settings');

/**
 * Registers settings, sections, and fields for the WC Integration page.
 */
function minichat_wc_addon_register_admin_settings() {

    $settings_group = 'minichat_wc_addon_settings_group';
    $page_slug = 'wc-minichat-settings';

    // --- Register Settings (Alle mit show_in_rest=false) ---
    register_setting($settings_group, MINICHAT_WC_ADDON_ENABLE_OPTION, [ 'type' => 'boolean', 'sanitize_callback' => 'rest_sanitize_boolean', 'default' => false, 'show_in_rest' => false, ]);
    register_setting($settings_group, 'minichat_wc_max_products', [ 'type' => 'integer', 'sanitize_callback' => 'absint', 'default' => 5, 'show_in_rest' => false, ]);
    register_setting($settings_group, 'minichat_wc_search_scope', [ 'type' => 'array', 'sanitize_callback' => 'minichat_wc_addon_sanitize_search_scope', 'default' => ['title' => true, 'description' => false, 'sku' => true, 'tags' => false], 'show_in_rest' => false, ]);
    register_setting($settings_group, 'minichat_wc_image_size', [ 'type' => 'string', 'sanitize_callback' => 'sanitize_key', 'default' => 'woocommerce_thumbnail', 'show_in_rest' => false, ]);
    register_setting($settings_group, 'minichat_wc_show_short_desc', [ 'type' => 'boolean', 'sanitize_callback' => 'rest_sanitize_boolean', 'default' => false, 'show_in_rest' => false, ]);

    // --- Add Settings Section ---
    add_settings_section( 'minichat_wc_addon_general_section', __('General Settings', 'wc-minichat-addon'), null, $page_slug );
    add_settings_section( 'minichat_wc_addon_search_section', __('Product Search & Display', 'wc-minichat-addon'), null, $page_slug );

    // --- Add Settings Fields ---
    add_settings_field( MINICHAT_WC_ADDON_ENABLE_OPTION . '_field', __('Enable Integration', 'wc-minichat-addon'), 'minichat_wc_addon_render_enable_field', $page_slug, 'minichat_wc_addon_general_section' );
    add_settings_field( 'minichat_wc_max_products_field', __('Max Products in Results', 'wc-minichat-addon'), 'minichat_wc_addon_render_max_products_field', $page_slug, 'minichat_wc_addon_search_section' );
    add_settings_field( 'minichat_wc_search_scope_field', __('Search Scope', 'wc-minichat-addon'), 'minichat_wc_addon_render_search_scope_field', $page_slug, 'minichat_wc_addon_search_section' );
    add_settings_field( 'minichat_wc_show_short_desc_field', __('Display Short Description', 'wc-minichat-addon'), 'minichat_wc_addon_render_show_short_desc_field', $page_slug, 'minichat_wc_addon_search_section' );
    add_settings_field( 'minichat_wc_image_size_field', __('Product Image Size', 'wc-minichat-addon'), 'minichat_wc_addon_render_image_size_field', $page_slug, 'minichat_wc_addon_search_section' );
}

/**
 * Sanitizes the search scope array.
 */
function minichat_wc_addon_sanitize_search_scope($input) {
    $allowed_keys = ['title', 'description', 'sku', 'tags']; $output = [];
    $defaults = ['title' => true, 'description' => false, 'sku' => true, 'tags' => false];
    if (!is_array($input)) { return $defaults; }
    foreach ($allowed_keys as $key) { $output[$key] = isset($input[$key]) ? rest_sanitize_boolean($input[$key]) : false; }
    return $output;
}


// --- Rendering Functions for Settings Fields (unverändert) ---

function minichat_wc_addon_render_enable_field() {
    $option_name = MINICHAT_WC_ADDON_ENABLE_OPTION; $option_value = get_option($option_name, false);
    ?> <label for="<?php echo esc_attr($option_name); ?>"><input type="checkbox" id="<?php echo esc_attr($option_name); ?>" name="<?php echo esc_attr($option_name); ?>" value="1" <?php checked(true, (bool)$option_value, true); ?>/> <?php esc_html_e('Enable WooCommerce product search and display for the Minimal Chatbot.', 'wc-minichat-addon'); ?></label><p class="description"><?php esc_html_e('If checked, the chatbot will attempt to answer product-related questions using your WooCommerce store data.', 'wc-minichat-addon'); ?></p> <?php
}
function minichat_wc_addon_render_max_products_field() {
    $option_name = 'minichat_wc_max_products'; $option_value = get_option($option_name, 5);
    ?> <input type="number" id="<?php echo esc_attr($option_name); ?>" name="<?php echo esc_attr($option_name); ?>" value="<?php echo esc_attr($option_value); ?>" min="1" max="20" step="1" class="small-text"/> <p class="description"><?php esc_html_e('Maximum number of products to show in the chat results (e.g., 1-10). Default: 5.', 'wc-minichat-addon'); ?></p> <?php
}
function minichat_wc_addon_render_search_scope_field() {
    $option_name = 'minichat_wc_search_scope'; $defaults = ['title' => true, 'description' => false, 'sku' => true, 'tags' => false]; $option_value = wp_parse_args(get_option($option_name, []), $defaults);
    $scopes = [ 'title' => __('Search in Product Titles', 'wc-minichat-addon'), 'description' => __('Search in Product Descriptions (Short & Long)', 'wc-minichat-addon'), 'sku' => __('Search in Product SKU', 'wc-minichat-addon'), 'tags' => __('Search in Product Tags', 'wc-minichat-addon'), ];
    ?> <fieldset><legend class="screen-reader-text"><span><?php _e('Search Scope Options', 'wc-minichat-addon'); ?></span></legend> <?php foreach ($scopes as $key => $label) : ?> <label for="<?php echo esc_attr($option_name . '_' . $key); ?>" style="display: block; margin-bottom: 5px;"><input type="checkbox" id="<?php echo esc_attr($option_name . '_' . $key); ?>" name="<?php echo esc_attr($option_name . '[' . $key . ']'); ?>" value="1" <?php checked(true, (bool)$option_value[$key], true); ?>/> <?php echo esc_html($label); ?></label> <?php endforeach; ?> </fieldset> <p class="description"><?php esc_html_e('Select where the chatbot should look for keywords when searching products. Searching descriptions can impact performance on large sites.', 'wc-minichat-addon'); ?></p> <?php
}
function minichat_wc_addon_render_show_short_desc_field() {
    $option_name = 'minichat_wc_show_short_desc'; $option_value = get_option($option_name, false);
    ?> <label for="<?php echo esc_attr($option_name); ?>"><input type="checkbox" id="<?php echo esc_attr($option_name); ?>" name="<?php echo esc_attr($option_name); ?>" value="1" <?php checked(true, (bool)$option_value, true); ?>/> <?php esc_html_e('Show product short description in the chat results card.', 'wc-minichat-addon'); ?></label> <p class="description"><?php esc_html_e('If checked, the short description will be included below the product title.', 'wc-minichat-addon'); ?></p> <?php
}
function minichat_wc_addon_render_image_size_field() {
     $option_name = 'minichat_wc_image_size'; $option_value = get_option($option_name, 'woocommerce_thumbnail');
     global $_wp_additional_image_sizes; $image_sizes = []; $default_image_sizes = get_intermediate_image_sizes();
     foreach ($default_image_sizes as $size) { $width = intval(get_option("{$size}_size_w")); $height = intval(get_option("{$size}_size_h")); $label = ucwords(str_replace(['_', '-'], ' ', $size)); if ($width > 0 && $height > 0) { $label .= " ({$width}x{$height})"; } elseif ($width > 0) { $label .= " ({$width}x?)"; } elseif ($height > 0) { $label .= " (?x{$height})"; } $image_sizes[$size] = $label; }
     if (is_array($_wp_additional_image_sizes) && count($_wp_additional_image_sizes)) { foreach ($_wp_additional_image_sizes as $size => $size_data) { if (!isset($image_sizes[$size])) { $width = intval($size_data['width']); $height = intval($size_data['height']); $label = ucwords(str_replace(['_', '-'], ' ', $size)); if ($width > 0 && $height > 0) { $label .= " ({$width}x{$height})"; } elseif ($width > 0) { $label .= " ({$width}x?)"; } elseif ($height > 0) { $label .= " (?x{$height})"; } $image_sizes[$size] = $label; } } }
     $image_sizes['full'] = __('Full Size', 'wc-minichat-addon') . ' (' . __('Original Uploaded', 'wc-minichat-addon') . ')';
     if (!array_key_exists($option_value, $image_sizes)) { $option_value = 'woocommerce_thumbnail'; }
    ?> <select id="<?php echo esc_attr($option_name); ?>" name="<?php echo esc_attr($option_name); ?>"> <?php foreach ($image_sizes as $slug => $label) : ?> <option value="<?php echo esc_attr($slug); ?>" <?php selected($option_value, $slug); ?>><?php echo esc_html($label); ?></option> <?php endforeach; ?> </select> <p class="description"><?php esc_html_e('Select the image size to display in the product cards. Smaller sizes like WooCommerce Thumbnail are recommended for performance.', 'wc-minichat-addon'); ?></p> <?php
}


// --- Admin Page Rendering Function ---

/**
 * Renders the HTML content for the WooCommerce Integration settings page.
 */
function minichat_wc_addon_render_settings_page() {
    // Check user capability
    if (!current_user_can('manage_woocommerce') && !current_user_can('manage_options')) {
        wp_die(__('You do not have sufficient permissions to access this page.'));
        return;
    }

    // Ensure the is_plugin_active function is available.
    if (!function_exists('is_plugin_active')) {
        include_once(ABSPATH . 'wp-admin/includes/plugin.php');
    }

    // --- Perform Status Checks ---
    $core_active = function_exists('is_plugin_active') ? is_plugin_active(MINICHAT_CORE_PLUGIN_SLUG) : false;
    $wc_active = function_exists('is_plugin_active') ? is_plugin_active(WOOCOMMERCE_PLUGIN_SLUG) : false;
    $integration_enabled = (bool) get_option(MINICHAT_WC_ADDON_ENABLE_OPTION, false);
    $is_ready = $core_active && $wc_active && $integration_enabled;

    // --- NEU: Anzahl veröffentlichter Produkte prüfen ---
    $published_product_count = -1; // Default: Check nicht durchgeführt oder fehlgeschlagen
    $product_check_error = '';
    // Führe den Check nur aus, wenn WooCommerce aktiv ist und die Funktion existiert
    if ($wc_active && function_exists('wc_get_products')) {
        try {
            // Einfache Abfrage, um nur die IDs aller veröffentlichten Produkte zu erhalten
            $product_ids = wc_get_products([
                'status' => 'publish', // Nur veröffentlichte
                'limit'  => -1,       // Keine Begrenzung
                'return' => 'ids',     // Nur die IDs zurückgeben (effizienter)
            ]);
            // Prüfen, ob das Ergebnis ein Array ist (sollte es sein)
            if (is_array($product_ids)) {
                $published_product_count = count($product_ids);
            } else {
                // Fallback, falls wc_get_products etwas anderes zurückgibt
                $published_product_count = 0;
                $product_check_error = __('Die Produktabfrage gab kein erwartetes Ergebnis zurück.', 'wc-minichat-addon');
            }
        } catch (Exception $e) {
             // Fehler während der Abfrage abfangen
             $published_product_count = 0; // Sicherer Wert bei Fehler
             $product_check_error = __('Fehler bei der Produktabfrage: ', 'wc-minichat-addon') . esc_html($e->getMessage());
        }
    } elseif ($wc_active && !function_exists('wc_get_products')) {
         $product_check_error = __('WooCommerce ist aktiv, aber die Funktion wc_get_products fehlt.', 'wc-minichat-addon');
    } else {
         $product_check_error = __('WooCommerce ist nicht aktiv.', 'wc-minichat-addon');
    }
    // --- Ende Produktprüfung ---


    ?>
    <div class="wrap" id="wc-minichat-addon-settings-page">
        <h1><?php esc_html_e('Minimal Chatbot - WooCommerce Integration', 'wc-minichat-addon'); ?></h1>
        <p><?php esc_html_e('Configure how the Minimal Chatbot interacts with your WooCommerce store.', 'wc-minichat-addon'); ?></p>

        <hr>
        <h2><?php esc_html_e('Integration Status', 'wc-minichat-addon'); ?></h2>
        <table class="form-table wc-minichat-status-table" role="presentation">
            <tbody>
                <tr>
                    <th scope="row"><?php esc_html_e('Minimal Chatbot Plugin', 'wc-minichat-addon'); ?></th>
                    <td>
                        <?php if ($core_active) : ?>
                            <span style="color: #28a745; font-weight: bold;"><?php esc_html_e('Active', 'wc-minichat-addon'); ?></span> <span class="dashicons dashicons-yes-alt" style="color: #28a745; vertical-align: text-bottom;"></span>
                        <?php else : ?>
                            <span style="color: #dc3545; font-weight: bold;"><?php esc_html_e('Inactive', 'wc-minichat-addon'); ?></span> <span class="dashicons dashicons-warning" style="color: #dc3545; vertical-align: text-bottom;"></span><br>
                            <small><?php printf(__('Please install and activate the main Minimal Chatbot plugin (Expected Path: %s).', 'wc-minichat-addon'), '<code>' . esc_html(MINICHAT_CORE_PLUGIN_SLUG) . '</code>'); ?></small>
                        <?php endif; ?>
                    </td>
                </tr>
                 <tr>
                    <th scope="row"><?php esc_html_e('WooCommerce Plugin', 'wc-minichat-addon'); ?></th>
                    <td>
                        <?php if ($wc_active) : ?>
                            <span style="color: #28a745; font-weight: bold;"><?php esc_html_e('Active', 'wc-minichat-addon'); ?></span> <span class="dashicons dashicons-yes-alt" style="color: #28a745; vertical-align: text-bottom;"></span>
                        <?php else : ?>
                            <span style="color: #dc3545; font-weight: bold;"><?php esc_html_e('Inactive', 'wc-minichat-addon'); ?></span> <span class="dashicons dashicons-warning" style="color: #dc3545; vertical-align: text-bottom;"></span><br>
                            <small><?php esc_html_e('Please install and activate the WooCommerce plugin.', 'wc-minichat-addon'); ?></small>
                        <?php endif; ?>
                    </td>
                </tr>
                 <tr>
                    <th scope="row"><?php esc_html_e('Integration Setting', 'wc-minichat-addon'); ?></th>
                    <td>
                        <?php if ($integration_enabled) : ?>
                            <span style="color: #28a745; font-weight: bold;"><?php esc_html_e('Enabled', 'wc-minichat-addon'); ?></span> <span class="dashicons dashicons-yes-alt" style="color: #28a745; vertical-align: text-bottom;"></span>
                        <?php else : ?>
                            <span style="color: #fd7e14; font-weight: bold;"><?php esc_html_e('Disabled', 'wc-minichat-addon'); ?></span> <span class="dashicons dashicons-warning" style="color: #fd7e14; vertical-align: text-bottom;"></span><br>
                            <small><?php esc_html_e('Check the "Enable Integration" setting below to activate.', 'wc-minichat-addon'); ?></small>
                        <?php endif; ?>
                    </td>
                </tr>
                <?php // --- NEUE ZEILE für Produktanzahl --- ?>
                <tr>
                    <th scope="row"><?php esc_html_e('Detected Products', 'wc-minichat-addon'); ?></th>
                    <td>
                        <?php if ($published_product_count >= 0) : // Check war erfolgreich (auch wenn 0 gefunden wurden) ?>
                            <span style="font-weight: bold;"><?php echo esc_html($published_product_count); ?></span>
                            <?php esc_html_e('published product(s) detected by basic query.', 'wc-minichat-addon'); ?>
                            <?php if ($published_product_count == 0 && $wc_active) : ?>
                                <span class="dashicons dashicons-warning" style="color: #fd7e14; vertical-align: text-bottom;" title="<?php esc_attr_e('This might indicate an issue if you actually have published products in your store.', 'wc-minichat-addon'); ?>"></span>
                            <?php elseif ($published_product_count > 0): ?>
                                <span class="dashicons dashicons-yes-alt" style="color: #28a745; vertical-align: text-bottom;"></span>
                            <?php endif; ?>
                            <?php if(!empty($product_check_error)): ?>
                                <br><small style="color:#dc3545;"><?php echo esc_html($product_check_error); ?></small>
                            <?php endif; ?>
                        <?php else : // Check konnte nicht durchgeführt werden (WC inaktiv etc.) ?>
                            <span style="color: #6c757d;"><?php esc_html_e('Check Not Performed', 'wc-minichat-addon'); ?></span><br>
                            <small><?php echo esc_html($product_check_error); ?></small>
                        <?php endif; ?>
                    </td>
                </tr>
                 <?php // --- Ende Produktanzahl --- ?>
                <tr style="border-top: 1px solid #eee;">
                    <th scope="row"><?php esc_html_e('Overall Status', 'wc-minichat-addon'); ?></th>
                    <td>
                        <?php if ($is_ready) : ?>
                             <strong style="color: #28a745;"><?php esc_html_e('Integration is active and ready.', 'wc-minichat-addon'); ?></strong> <span class="dashicons dashicons-yes" style="color: #28a745; vertical-align: text-bottom;"></span>
                        <?php elseif (!$core_active || !$wc_active) : ?>
                             <strong style="color: #dc3545;"><?php esc_html_e('Integration inactive due to missing required plugins.', 'wc-minichat-addon'); ?></strong>
                        <?php else : // Core and WC active, but setting disabled ?>
                             <strong style="color: #fd7e14;"><?php esc_html_e('Integration is currently disabled in settings.', 'wc-minichat-addon'); ?></strong>
                        <?php endif; ?>
                    </td>
                </tr>
            </tbody>
        </table>
        <style>
            .wc-minichat-status-table td span { margin-right: 5px; }
            .wc-minichat-status-table td .dashicons { vertical-align: middle; /* Adjust alignment */ }
        </style>

        <hr>

        <form action="options.php" method="post">
            <?php
            // Output security fields for the registered setting group
            settings_fields('minichat_wc_addon_settings_group');

            // Output the settings sections and their fields for this page
            do_settings_sections('wc-minichat-settings'); // Use the page slug

            // Add the submit button
            submit_button(__('Save WooCommerce Settings', 'wc-minichat-addon'));
            ?>
        </form>
    </div><?php // Closing div for wrap
} // Closing tag for the function minichat_wc_addon_render_settings_page

?>