<?php
/**
 * AJAX Handler Functions for Minimal Chatbot
 *
 * Handles requests sent from the frontend JavaScript.
 * Included by the main plugin file. Assumes constants and helper functions are available.
 *
 * @package Minimal_Chatbot
 * @version 1.13.0 (Refactored for OpenAI Tool Calling for WC)
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// --- AJAX Handler for Chat Messages (Refactored for Tool Calling) ---
add_action('wp_ajax_minichat_send_message', 'minichat_handle_ajax_message');
add_action('wp_ajax_nopriv_minichat_send_message', 'minichat_handle_ajax_message');
function minichat_handle_ajax_message() {
    $start_time = microtime(true);
    $current_version = defined('MINICHAT_VERSION') ? MINICHAT_VERSION : '1.13.0';
    if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX v".esc_html($current_version)."]: === Handle Send Message Request (Tool Calling Active) ===");

    $new_uid = null;
    if (session_status() == PHP_SESSION_NONE && !headers_sent()) {
        session_start();
    }

    // 1. Security Check (Nonce)
    if (!check_ajax_referer('minichat_nonce', 'nonce', false)) {
        wp_send_json_error(['error' => __('Security check failed.', 'minimal-chatbot')], 403); return;
    }

    // 2. Prerequisite Checks (License, DB, API Key)
    $lic_ok = function_exists('minichat_is_license_valid') ? minichat_is_license_valid() : false;
    if (!$lic_ok) { wp_send_json_error(['error' => __('Chatbot license is invalid or trial has expired.', 'minimal-chatbot')]); return; }

    $db_ready_option_key = defined('MINICHAT_DB_READY_OPTION') ? MINICHAT_DB_READY_OPTION : 'minichat_db_ready_status_v1_6_2';
    $db_ok = (get_option($db_ready_option_key) === 'yes');
    if (!$db_ok) { wp_send_json_error(['error' => __('Chatbot database is not configured correctly.', 'minimal-chatbot')]); return; }

    $api_key = get_option('minichat_api_key');
    if (empty($api_key) || $api_key === 'DEIN_OPENAI_API_KEY_HIER') { // Prüfe auch auf Platzhalter
         wp_send_json_error(['error' => __('OpenAI API Key is missing or not configured.', 'minimal-chatbot')]); return;
    }
    $openai_model = get_option('minichat_openai_model', 'gpt-4o'); // Model aus Einstellungen holen

    // 3. Get User Info & Message
    if (!function_exists('minichat_get_persistent_user_id')) { wp_send_json_error(['error' => __('Internal error: Cannot identify user.', 'minimal-chatbot')]); return; }
    $uid = minichat_get_persistent_user_id();
    if (empty($uid)) {
        $uid = wp_generate_uuid4(); $new_uid = $uid;
        $session_key_name = defined('MINICHAT_USER_SESSION_KEY') ? MINICHAT_USER_SESSION_KEY : 'minichat_user_id_val';
        if (session_status() == PHP_SESSION_ACTIVE) { $_SESSION[$session_key_name] = $uid; }
    }
    $sid = (session_status() == PHP_SESSION_ACTIVE) ? session_id() : ('no_session_' . time());
    $umsg_raw = isset($_POST['message']) ? wp_unslash($_POST['message']) : '';
    $umsg = sanitize_text_field($umsg_raw);
    if (empty($umsg)) { wp_send_json_error(['error' => __('Message cannot be empty.', 'minimal-chatbot')]); return; }

    // 4. Trial Message Limit Check
    $is_trial = function_exists('minichat_is_trial_active') ? minichat_is_trial_active() : false;
    if ($is_trial) {
        global $wpdb; $tbl = $wpdb->prefix . 'minichat_history';
        if ($wpdb instanceof wpdb) {
            $mc = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM {$tbl} WHERE user_id = %s AND role='user'", $uid));
            $limit = defined('MINICHAT_MESSAGES_LIMIT_TRIAL') ? MINICHAT_MESSAGES_LIMIT_TRIAL : 50;
            if ($mc !== null && $mc >= $limit) { wp_send_json_error(['error' => sprintf(__('Trial message limit reached (%d messages).', 'minimal-chatbot'), $limit)]); return; }
        } else { wp_send_json_error(['error' => __('Database error during trial check.', 'minimal-chatbot')]); return; }
    }

    // --- NEUER ABLAUF: Tool Calling ---

    // 5. Konversationsverlauf vorbereiten (ohne Seiten-Kontext!)
    if (!function_exists('minichat_get_db_history')) { wp_send_json_error(['error' => __('Internal error: Cannot retrieve history.', 'minimal-chatbot')]); return; }
    $memory_depth = intval(get_option('minichat_memory_depth', 10));
    $history_messages = minichat_get_db_history($uid, $memory_depth); // Holt nur die letzten Nachrichten

    // System Prompt - Wichtig: Muss das Werkzeug erklären!
    $site_n = get_bloginfo('name');
    $site_d = get_bloginfo('description');
    $base_ctx = "Website Name: " . $site_n . ". Website Description: " . $site_d . ". ";
    $sys_prompt_setting = get_option('minichat_system_prompt', 'You are a helpful assistant.');
    // Ergänze den System Prompt um die Werkzeug-Info
    $sys_prompt_full = $base_ctx . $sys_prompt_setting . " You have access to a tool called 'produktSucheImShop' to search the live WooCommerce product database for products, prices, stock status etc. Use it whenever the user asks about products available in the shop, specific product details, prices, stock, or the general assortment. Decide which parameters ('suchbegriff', 'kategorie_slug', 'alle_auflisten') are needed based on the user's query. Use 'alle_auflisten=true' for general product listing requests.";

    $messages = [];
    $messages[] = ['role' => 'system', 'content' => $sys_prompt_full];
    if (!empty($history_messages)) {
        $messages = array_merge($messages, $history_messages);
    }
    $messages[] = ['role' => 'user', 'content' => $umsg]; // Aktuelle Nachricht hinzufügen

    // 6. Werkzeug-Definition für die API
    $tools = [
        [
            'type' => 'function',
            'function' => [
                'name' => 'produktSucheImShop', // Muss exakt zum Funktionsnamen im Tool-Call passen
                'description' => __('Searches the WooCommerce shop for products based on criteria. Can also list all products.', 'minimal-chatbot'), // Beschreibung für die KI
                'parameters' => [
                    'type' => 'object',
                    'properties' => [
                        'suchbegriff' => [
                            'type' => 'string',
                            'description' => __('Optional search terms for product titles, SKUs, descriptions.', 'minimal-chatbot')
                        ],
                        'kategorie_slug' => [
                            'type' => 'string',
                            'description' => __('Optional product category slug to filter the search (e.g., "t-shirts", "hoodies").', 'minimal-chatbot')
                        ],
                        'alle_auflisten' => [
                            'type' => 'boolean',
                            'description' => __('If true, lists all available products, ignoring other search parameters. Defaults to false.', 'minimal-chatbot')
                        ],
                        // Zukünftig: 'min_preis', 'max_preis', 'attribute' => ['type' => 'object', 'properties' => ['pa_color'=>['type'=>'string'], ...]]
                    ],
                    'required' => [], // Keine Parameter sind zwingend
                ],
            ]
        ]
    ];

    // 7. Erster API-Aufruf (Prüfen, ob Tool genutzt werden soll)
    if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: Sending initial request to OpenAI. Message count: " . count($messages));
    $api_url = 'https://api.openai.com/v1/chat/completions';
    $api_args_1 = [
        'method' => 'POST',
        'headers' => [ 'Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $api_key ],
        'body' => json_encode([
            'model' => $openai_model,
            'messages' => $messages,
            'tools' => $tools,
            'tool_choice' => 'auto',
             // Ggf. max_tokens hier begrenzen, falls Antwort erwartet wird
             'max_tokens' => intval(get_option('minichat_max_tokens', 1000)), // Etwas höher für potenzielle Tool-Calls
             'temperature' => floatval(get_option('minichat_temperature', 0.7)),
        ]),
        'timeout' => intval(get_option('minichat_api_timeout', 45)) // Höherer Timeout
    ];

    $response_1 = wp_remote_post($api_url, $api_args_1);

    if (is_wp_error($response_1)) { wp_send_json_error(['error' => 'API Call 1 Error: ' . $response_1->get_error_message()]); return; }
    $body_1 = wp_remote_retrieve_body($response_1);
    $data_1 = json_decode($body_1, true);
    $http_code_1 = wp_remote_retrieve_response_code($response_1);

    if ($http_code_1 !== 200 || !$data_1 || !isset($data_1['choices'][0]['message'])) { wp_send_json_error(['error' => 'API Call 1 Error: Invalid response. Code: ' . $http_code_1]); return; }

    $ai_message = $data_1['choices'][0]['message'];
    $final_reply = '';
    $structured_data_for_frontend = []; // Initialisieren

    // 8. Prüfen, ob Werkzeug aufgerufen wurde
    if (isset($ai_message['tool_calls']) && !empty($ai_message['tool_calls'])) {
         if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: OpenAI requested tool call: " . print_r($ai_message['tool_calls'], true));

         // Hänge die Aufforderung der KI zum Verlauf hinzu
         $messages[] = $ai_message; // Wichtig für den Kontext des zweiten Calls

         // Prüfe, ob die WC Integration überhaupt aktiviert ist und die Funktion existiert
         $wc_integration_enabled = get_option(MINICHAT_WC_ADDON_ENABLE_OPTION, false);
         if (!$wc_integration_enabled || !function_exists('minichat_wc_tool_execute_search')) {
              if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: Tool requested, but WC integration disabled or function missing.");
              // Sende eine Standardantwort oder Fehlermeldung zurück an die KI im zweiten Call
               $tool_response_content = "Fehler: Die Produktsuche ist derzeit nicht verfügbar.";
               // Füge die (Fehler-)Antwort des Werkzeugs zum Verlauf hinzu
                foreach ($ai_message['tool_calls'] as $tool_call) { // Annahme: Nur ein relevanter Call
                    if ($tool_call['type'] === 'function' && $tool_call['function']['name'] === 'produktSucheImShop') {
                         $messages[] = ['role' => 'tool', 'tool_call_id' => $tool_call['id'], 'name' => 'produktSucheImShop', 'content' => $tool_response_content];
                         break; // Nur den ersten relevanten Call behandeln
                    }
                }
         } else {
              // Führe die Werkzeugaufrufe aus (normalerweise nur einer)
              foreach ($ai_message['tool_calls'] as $tool_call) {
                 if ($tool_call['type'] === 'function' && $tool_call['function']['name'] === 'produktSucheImShop') {
                     $tool_call_id = $tool_call['id'];
                     // Dekodiere die von der KI vorgeschlagenen Argumente
                     $arguments = isset($tool_call['function']['arguments']) ? json_decode($tool_call['function']['arguments'], true) : [];
                     if (json_last_error() !== JSON_ERROR_NONE) {
                          if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: ERROR - Failed to decode JSON arguments from AI: " . $tool_call['function']['arguments']);
                          $tool_response_content = "Fehler: Ungültige Argumente vom KI-Modell erhalten.";
                          $structured_data_for_frontend = [];
                     } else {
                          if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: Calling 'minichat_wc_tool_execute_search' with args: " . print_r($arguments, true));
                          // Rufe die Suchfunktion aus woocommerce-integration.php auf
                          $tool_result = minichat_wc_tool_execute_search($arguments);
                          // Extrahiere Text für KI und strukturierte Daten für Frontend
                          $tool_response_content = isset($tool_result['text_for_ai']) ? $tool_result['text_for_ai'] : 'Fehler bei der Produktsuche.';
                          $structured_data_for_frontend = isset($tool_result['structured_data']) && is_array($tool_result['structured_data']) ? $tool_result['structured_data'] : [];
                          if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: Tool execution finished. AI Text Length: " . strlen($tool_response_content) . ", Structured Data Count: " . count($structured_data_for_frontend));
                     }

                     // Füge das Ergebnis des Werkzeugs zum Verlauf hinzu
                     $messages[] = [
                         'role' => 'tool',
                         'tool_call_id' => $tool_call_id,
                         'name' => 'produktSucheImShop',
                         'content' => $tool_response_content // String-Ergebnis der Suche
                     ];
                     break; // Verarbeite nur den ersten passenden Tool Call
                 }
              } // Ende foreach tool_calls
         } // Ende else (WC Integration enabled)

        // 9. Zweiter API-Aufruf mit den Werkzeug-Ergebnissen
         if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: Sending second request to OpenAI with tool results. Message count: " . count($messages));
         $api_args_2 = [
             'method' => 'POST',
             'headers' => [ 'Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $api_key ],
             'body' => json_encode([
                 'model' => $openai_model,
                 'messages' => $messages, // Send updated history
                 'max_tokens' => intval(get_option('minichat_max_tokens', 1000)), // Use regular token limit
                 'temperature' => floatval(get_option('minichat_temperature', 0.7)),
                 // Kein 'tools' oder 'tool_choice' mehr hier!
             ]),
             'timeout' => intval(get_option('minichat_api_timeout', 30)) // Normaler Timeout
         ];

         $response_2 = wp_remote_post($api_url, $api_args_2);

         if (is_wp_error($response_2)) { wp_send_json_error(['error' => 'API Call 2 Error: ' . $response_2->get_error_message()]); return; }
         $body_2 = wp_remote_retrieve_body($response_2);
         $data_2 = json_decode($body_2, true);
         $http_code_2 = wp_remote_retrieve_response_code($response_2);

         if ($http_code_2 !== 200 || !$data_2 || !isset($data_2['choices'][0]['message']['content'])) { wp_send_json_error(['error' => 'API Call 2 Error: Invalid response. Code: ' . $http_code_2]); return; }

         // Finale Antwort der KI
         $final_reply = $data_2['choices'][0]['message']['content'];
         if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: Received final reply from OpenAI after tool use.");

    } else {
         // KI hat direkt geantwortet ohne Werkzeug
         $final_reply = isset($ai_message['content']) ? $ai_message['content'] : __('An error occurred processing the request.', 'minimal-chatbot');
         $structured_data_for_frontend = []; // Keine Produktdaten ohne Tool-Aufruf
         if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: OpenAI responded directly without tool use.");
    }

    // 10. Finale Antwort verarbeiten und speichern
    $final_reply = trim($final_reply);
    if (empty($final_reply)) {
         $final_reply = __('[Empty response received from AI]', 'minimal-chatbot');
          if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: WARN - Final AI reply was empty.");
    }

    // 11. In DB speichern
    $user_msg_id = 0; $bot_msg_id = 0;
    if (function_exists('minichat_save_db_message')) {
         $user_msg_id = minichat_save_db_message($uid, $sid, 'user', $umsg);
         $bot_msg_id = minichat_save_db_message($uid, $sid, 'assistant', $final_reply); // Speichere die finale KI-Antwort
         if (!$user_msg_id || !$bot_msg_id) { if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: WARN Failed to save messages to DB."); }
         else { if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: Saved messages to DB. User ID: {$user_msg_id}, Bot ID: {$bot_msg_id}"); }
    } else { if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: ERR Cannot save conversation - minichat_save_db_message missing."); }

    // 12. JSON-Antwort für Frontend vorbereiten
    $response_data = [
        'reply' => $final_reply,
        'message_id' => $bot_msg_id
    ];
    // Füge neue User-ID hinzu, falls generiert
    if ($new_uid) {
        $response_data['new_user_id'] = $new_uid;
    }
    // Füge die strukturierten Produktdaten hinzu, falls vorhanden (für JS/Kartenanzeige)
    if (!empty($structured_data_for_frontend)) {
         $response_data['minichat_wc_products'] = $structured_data_for_frontend;
          if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[AJAX Tool Call]: Adding " . count($structured_data_for_frontend) . " structured products to final JSON response.");
    }

    // Optional: Filter für finale Antwort (falls andere Add-ons eingreifen wollen)
    // $response_data = apply_filters('minichat_final_ajax_response', $response_data, $final_reply, $umsg, $uid);

    // 13. Antwort senden
    if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) {
        $end_time = microtime(true);
        error_log("MC[AJAX Tool Call]: === Handle Send Message Request END (Duration: " . round($end_time - $start_time, 3) . "s) ===");
    }
    wp_send_json_success($response_data);
}


// --- Die anderen AJAX Handler (Feedback, Clear Chat, License etc.) bleiben exakt wie vorher ---

// --- AJAX Handler for Feedback ---
add_action('wp_ajax_minichat_send_feedback', 'minichat_handle_ajax_feedback');
add_action('wp_ajax_nopriv_minichat_send_feedback', 'minichat_handle_ajax_feedback');
function minichat_handle_ajax_feedback() {
    check_ajax_referer('minichat_feedback_nonce', 'nonce');
    $msg_id = isset($_POST['message_id']) ? absint($_POST['message_id']) : 0;
    $rating = isset($_POST['rating']) ? intval($_POST['rating']) : null;
    $text = isset($_POST['feedback_text']) ? sanitize_textarea_field(wp_unslash($_POST['feedback_text'])) : '';
    if (empty($msg_id) || $rating === null || !in_array($rating, [0, 1])) { wp_send_json_error(['message' => __('Invalid feedback data provided.', 'minimal-chatbot')], 400); }
    global $wpdb; $table_name = $wpdb->prefix . 'minichat_history';
    if ($wpdb->get_var($wpdb->prepare("SHOW TABLES LIKE %s", $table_name)) !== $table_name) { wp_send_json_error(['message' => __('Database error occurred.', 'minimal-chatbot')], 500); }
    $updated_rows = $wpdb->update($table_name, ['feedback_rating' => $rating, 'feedback_text' => $text], ['id' => $msg_id, 'role' => 'assistant'], ['%d', '%s'], ['%d', '%s']);
    if ($updated_rows === false) { wp_send_json_error(['message' => __('Failed to save feedback due to a database issue.', 'minimal-chatbot')], 500); }
    elseif ($updated_rows === 0) {
        $message_exists = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM {$table_name} WHERE id = %d AND role = 'assistant'", $msg_id));
        if ($message_exists) { wp_send_json_success(['message' => __('Feedback saved!', 'minimal-chatbot')]); }
        else { wp_send_json_error(['message' => __('Feedback could not be saved. Message not found.', 'minimal-chatbot')], 404); }
    } else { wp_send_json_success(['message' => __('Feedback saved!', 'minimal-chatbot')]); }
}

// --- AJAX Handler for Clearing Chat History ---
add_action('wp_ajax_minichat_clear_chat', 'minichat_handle_ajax_clear_chat');
add_action('wp_ajax_nopriv_minichat_clear_chat', 'minichat_handle_ajax_clear_chat');
function minichat_handle_ajax_clear_chat() {
     check_ajax_referer('minichat_clear_chat_nonce', 'nonce');
     if (!function_exists('minichat_get_persistent_user_id')) { wp_send_json_error(['message' => __('Internal error: Cannot identify user.', 'minimal-chatbot')], 500); return; }
     $user_id = minichat_get_persistent_user_id();
     if (empty($user_id)) { wp_send_json_error(['message' => __('Could not identify user to clear history.', 'minimal-chatbot')], 400); return; }
     global $wpdb; $table_name = $wpdb->prefix . 'minichat_history';
     if ($wpdb->get_var($wpdb->prepare("SHOW TABLES LIKE %s", $table_name)) !== $table_name) { wp_send_json_error(['message' => __('Database error: History table not found.', 'minimal-chatbot')], 500); return; }
     $deleted_rows = $wpdb->delete( $table_name, ['user_id' => $user_id], ['%s']);
     if ($deleted_rows === false) { wp_send_json_error(['message' => __('Failed to clear history due to a database issue.', 'minimal-chatbot')], 500); }
     else { wp_send_json_success(['message' => __('Chat history cleared.', 'minimal-chatbot')]); }
}

// --- AJAX Handler for Trial Activation ---
add_action('wp_ajax_minichat_activate_trial', 'minichat_handle_ajax_activate_trial');
function minichat_handle_ajax_activate_trial() {
    check_ajax_referer('minichat_license_nonce', 'nonce');
    if (!current_user_can('manage_options')) { wp_send_json_error(['message' => __('You do not have permission to perform this action.', 'minimal-chatbot')], 403); return; }
    if (!function_exists('minichat_activate_trial')) { wp_send_json_error(['message' => __('Trial activation function is not available. Plugin files might be missing.', 'minimal-chatbot')], 500); return; }
    $result = minichat_activate_trial();
    if (isset($result['success']) && $result['success'] === true) { wp_send_json_success($result); }
    else { $error_message = isset($result['message']) ? $result['message'] : __('Unknown error during trial activation.', 'minimal-chatbot'); wp_send_json_error(['message' => $error_message]); }
}


// --- License AJAX Handlers ---
add_action('wp_ajax_minichat_activate_license', 'minichat_handle_ajax_activate_license');
function minichat_handle_ajax_activate_license() {
     check_ajax_referer('minichat_license_nonce', 'nonce');
     if (!current_user_can('manage_options')) { wp_send_json_error(['message' => __('Permission denied.', 'minimal-chatbot')], 403); return; }
     $license_key = isset($_POST['license_key']) ? sanitize_text_field(trim($_POST['license_key'])) : '';
     if (empty($license_key)) { wp_send_json_error(['message' => __('License key cannot be empty.', 'minimal-chatbot')], 400); return; }
     if (!function_exists('minichat_activate_license_remote')) { wp_send_json_error(['message' => __('License activation function is not available.', 'minimal-chatbot')], 500); return; }
     $result = minichat_activate_license_remote($license_key);
     if (isset($result['error'])) { wp_send_json_error(['message' => $result['error']]); }
     else { wp_send_json_success($result); }
}

add_action('wp_ajax_minichat_verify_license', 'minichat_handle_ajax_verify_license');
function minichat_handle_ajax_verify_license() {
    check_ajax_referer('minichat_license_nonce', 'nonce');
    if (!current_user_can('manage_options')) { wp_send_json_error(['message' => __('Permission denied.', 'minimal-chatbot')], 403); return; }
    $license_options_key = defined('MINICHAT_LICENSE_OPTIONS_NAME') ? MINICHAT_LICENSE_OPTIONS_NAME : 'minichat_license_options'; $options = get_option($license_options_key, []); $license_key = isset($options['key']) ? $options['key'] : '';
    if (empty($license_key)) { wp_send_json_error(['message' => __('No license key is currently saved to verify.', 'minimal-chatbot')], 400); return; }
    if (!function_exists('minichat_verify_license_remote')) { wp_send_json_error(['message' => __('License verification function is not available.', 'minimal-chatbot')], 500); return; }
    $result = minichat_verify_license_remote($license_key);
    if (isset($result['error'])) { wp_send_json_error(['message' => $result['error']]); }
    else { wp_send_json_success($result); }
}

// --- AJAX Handler for Context View in Feedback ---
add_action('wp_ajax_minichat_get_message_context', 'minichat_handle_ajax_get_context');
function minichat_handle_ajax_get_context() {
    check_ajax_referer('minichat_context_nonce', 'nonce');
    if (!current_user_can('manage_options')) { wp_send_json_error(['message' => __('Permission denied.', 'minimal-chatbot')], 403); return; }
    $message_id = isset($_POST['message_id']) ? absint($_POST['message_id']) : 0; $user_id    = isset($_POST['user_id']) ? sanitize_text_field($_POST['user_id']) : ''; $session_id = isset($_POST['session_id']) ? sanitize_text_field($_POST['session_id']) : '';
    if (!$message_id || empty($user_id) || empty($session_id)) { wp_send_json_error(['message' => __('Missing required parameters (message_id, user_id, session_id).', 'minimal-chatbot')], 400); return; }
    global $wpdb; $table_name = $wpdb->prefix . 'minichat_history';
    if ($wpdb->get_var($wpdb->prepare("SHOW TABLES LIKE %s", $table_name)) !== $table_name) { wp_send_json_error(['message' => __('Database error: History table not found.', 'minimal-chatbot')], 500); return; }
    $memory_depth_setting = intval(get_option('minichat_memory_depth', 10)); $limit = max(1, $memory_depth_setting) * 2 + 5;
    $context_messages = $wpdb->get_results( $wpdb->prepare( "SELECT role, content, timestamp, id FROM {$table_name} WHERE user_id = %s AND session_id = %s AND id <= %d ORDER BY id DESC LIMIT %d", $user_id, $session_id, $message_id, $limit ), ARRAY_A );
    if ($wpdb->last_error) { wp_send_json_error(['message' => __('Database error while fetching context.', 'minimal-chatbot')], 500); return; } if (empty($context_messages)) { wp_send_json_error(['message' => __('No context found for this message.', 'minimal-chatbot')], 404); return; }
    $context_messages = array_reverse($context_messages); $html = '<p style="font-size:small; color:#666;">'.__('Showing conversation context leading up to the selected message (ID:','minimal-chatbot') . ' ' . esc_html($message_id) . '):</p>'; $html .= '<dl style="max-height: 400px; overflow-y: auto; padding: 10px; border: 1px solid #eee; background: #fdfdfd; font-size: 13px;">';
    foreach ($context_messages as $msg) { $is_target_message = ($msg['id'] == $message_id); $role_label = ($msg['role'] === 'user') ? __('User', 'minimal-chatbot') : __('Bot', 'minimal-chatbot'); $time_formatted = esc_html(wp_date(get_option('time_format'), strtotime($msg['timestamp']))); $dt_style = 'font-weight: bold; margin-bottom: 2px;' . ($is_target_message ? ' color: #c00;' : ''); $dd_style = 'margin-left: 15px; margin-bottom: 12px; padding: 8px; border-radius: 4px; background-color: ' . ($msg['role'] === 'user' ? '#f0f5fa' : '#f9f9f9') . '; border: 1px solid ' . ($is_target_message ? '#f5c6cb' : '#eee') . ';' . ($is_target_message ? ' background-color: #fff8f8;' : ''); $html .= '<dt style="' . esc_attr($dt_style) . '">' . esc_html($role_label) . ' (' . $time_formatted . '):</dt>'; $html .= '<dd style="' . esc_attr($dd_style) . '">' . nl2br(esc_html($msg['content'])) . '</dd>'; } $html .= '</dl>';
    wp_send_json_success(['html' => $html]);
}

?>