<?php
// includes/class-minichat-users-list-table.php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Load WP_List_Table if not already loaded
if (!class_exists('WP_List_Table')) {
    require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}

class Minichat_Users_List_Table extends WP_List_Table {

    private $history_table_name;

    function __construct() {
        global $wpdb;
        $this->history_table_name = $wpdb->prefix . 'minichat_history';

        parent::__construct([
            'singular' => __('User', 'minimal-chatbot'),   // Singular name of the listed records
            'plural'   => __('Users', 'minimal-chatbot'),  // Plural name of the listed records
            'ajax'     => false                             // Does this table support ajax?
        ]);
    }

    /**
	 * Retrieve users data from the database
	 *
	 * @param int $per_page
	 * @param int $page_number
	 *
	 * @return mixed
	 */
	public static function get_users_data( $per_page = 20, $page_number = 1 ) {
        global $wpdb;
        $table_name = $wpdb->prefix . 'minichat_history';

        $sql = "SELECT
                    user_id,
                    COUNT(*) as message_count,
                    MIN(timestamp) as first_seen,
                    MAX(timestamp) as last_seen
                FROM {$table_name}
                GROUP BY user_id";

        // Ordering
        $orderby = isset($_REQUEST['orderby']) ? sanitize_sql_orderby($_REQUEST['orderby']) : 'last_seen'; // Default order by last_seen
        $order = isset($_REQUEST['order']) ? strtoupper(sanitize_key($_REQUEST['order'])) : 'DESC'; // Default order DESC

        // Validate orderby input against allowed columns
        $allowed_orderby = ['user_id', 'message_count', 'first_seen', 'last_seen'];
        if (!in_array($orderby, $allowed_orderby)) {
             $orderby = 'last_seen';
        }
         if (!in_array($order, ['ASC', 'DESC'])) {
            $order = 'DESC';
        }

        $sql .= ' ORDER BY ' . $orderby . ' ' . $order;
        $sql .= " LIMIT $per_page";
		$sql .= ' OFFSET ' . ( $page_number - 1 ) * $per_page;

		$result = $wpdb->get_results( $sql, 'ARRAY_A' );
        if ($wpdb->last_error) {
             if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) error_log("MC[UsersTable]: DB Error getting users: " . $wpdb->last_error);
             return []; // Return empty array on error
        }

		return $result;
	}

    /**
	 * Returns the count of records in the database.
	 *
	 * @return null|string
	 */
	public static function record_count() {
		global $wpdb;
        $table_name = $wpdb->prefix . 'minichat_history';

        // Query to count distinct users
		$sql = "SELECT COUNT(DISTINCT user_id) FROM {$table_name}";

		return $wpdb->get_var( $sql );
	}


    /** Text displayed when no user data is available */
	public function no_items() {
		_e( 'No users found in chat history.', 'minimal-chatbot' );
	}

    /**
	 * Render a column when no column specific method exist.
	 *
	 * @param array $item
	 * @param string $column_name
	 *
	 * @return mixed
	 */
	public function column_default( $item, $column_name ) {
		switch ( $column_name ) {
			case 'user_id':
                // Display shortened user ID
                return '...' . esc_html(substr($item['user_id'], -12));
			case 'message_count':
                return intval($item['message_count']);
            case 'first_seen':
                return esc_html(wp_date(get_option('date_format') . ' H:i', strtotime($item['first_seen'])));
            case 'last_seen':
                 return esc_html(wp_date(get_option('date_format') . ' H:i', strtotime($item['last_seen'])));
            default:
				return print_r( $item, true ); //Show the whole array for troubleshooting purposes
		}
	}

    /**
	 * Method for user_id column with actions
	 *
	 * @param array $item an array of DB data
	 *
	 * @return string
	 */
	function column_user_id( $item ) {
        $user_id_display = '...' . esc_html(substr($item['user_id'], -12));

        // Add row actions
        $actions = array(
			// Example action: Link to view full history for this user (requires new page/logic)
            /*
            'view_history' => sprintf(
                '<a href="?page=%s&action=view_user_history&user_id=%s">View History</a>',
                 'minimal-chatbot-user-history', // A potential new page slug
                 esc_attr($item['user_id'])
            )
            */
		);

		return $user_id_display . $this->row_actions( $actions );
	}


    /**
	 * Associative array of columns
	 *
	 * @return array
	 */
	function get_columns() {
		$columns = [
			//'cb'      => '<input type="checkbox" />', // If bulk actions needed
			'user_id'    => __( 'User ID (End)', 'minimal-chatbot' ),
			'message_count' => __( 'Messages', 'minimal-chatbot' ),
			'first_seen'  => __( 'First Seen', 'minimal-chatbot' ),
			'last_seen' => __( 'Last Seen', 'minimal-chatbot' ),
		];
		return $columns;
	}

    /**
	 * Columns to make sortable.
	 *
	 * @return array
	 */
	public function get_sortable_columns() {
		$sortable_columns = array(
			'message_count' => array( 'message_count', false ),
			'first_seen' => array( 'first_seen', false ),
            'last_seen' => array( 'last_seen', true ), // Default sort by last_seen DESC
		);
		return $sortable_columns;
	}

    /**
	 * Handles data query and filter, sorting, and pagination.
	 */
	public function prepare_items() {
		$this->_column_headers = $this->get_column_info();

		/** Process bulk action */
		//$this->process_bulk_action(); // Add if needed later

		$per_page     = $this->get_items_per_page( 'users_per_page', 20 );
		$current_page = $this->get_pagenum();
		$total_items  = self::record_count();

		$this->set_pagination_args( [
			'total_items' => $total_items,                     //WE have to calculate the total number of items
			'per_page'    => $per_page                         //WE have to determine how many items to show on a page
		] );

		$this->items = self::get_users_data( $per_page, $current_page );
	}

    // Add process_bulk_action() method here if you implement bulk actions later
    /*
    public function process_bulk_action() {
        // Detect when a bulk action is being triggered...
        // if ('delete' === $this->current_action()) {
        //   // Verify the nonce
        //   // Process delete action
        // }
    }
    */
}