<?php
/**
 * Helper Functions for Minimal Chatbot
 *
 * Contains various utility functions used throughout the plugin.
 * Included by the main plugin file. Assumes constants and WordPress functions are available.
 *
 * @package Minimal_Chatbot
 * @version 1.11.12
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// --- Cookie/Session/User ID Helper ---

/**
 * Retrieves the persistent user ID, checking cookie first, then session.
 * NOTE: This function might attempt to start a session if needed and headers aren't sent.
 *
 * @return string|null The user ID string or null if not found.
 */
function minichat_get_persistent_user_id() {
    $cookie_name = defined('MINICHAT_USER_COOKIE_NAME') ? MINICHAT_USER_COOKIE_NAME : 'minichat_user_id';
    $session_key = 'minichat_user_id_val'; // Internal session key name
    $debug_log = defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG;

    // 1. Check Cookie
    if (isset($_COOKIE[$cookie_name]) && !empty($_COOKIE[$cookie_name])) {
        $pattern = '/[^a-fA-F0-9-]/'; // Allow UUID characters
        $cookie_id = preg_replace($pattern, '', $_COOKIE[$cookie_name]);
        // Basic length check for UUID-like format
        if (!empty($cookie_id) && strlen($cookie_id) > 10) {
             if ($debug_log) error_log("MC[GetUserID]: Using Cookie ID: " . substr($cookie_id, -6));
            return $cookie_id;
        } else {
             if ($debug_log) error_log("MC[GetUserID]: Cookie found but invalid after sanitize: " . $_COOKIE[$cookie_name]);
        }
    }

    // 2. Check Session (start if needed and possible)
    if (session_status() == PHP_SESSION_NONE && !headers_sent()) {
        session_start();
        if ($debug_log) error_log("MC[GetUserID]: Session started within getter.");
    }

    if (session_status() == PHP_SESSION_ACTIVE && isset($_SESSION[$session_key]) && !empty($_SESSION[$session_key])) {
        $session_id_val = $_SESSION[$session_key];
        // Basic validation for session value as well
        $pattern = '/[^a-fA-F0-9-]/';
        $session_id_val = preg_replace($pattern, '', $session_id_val);
         if (!empty($session_id_val) && strlen($session_id_val) > 10) {
            if ($debug_log) error_log("MC[GetUserID]: Using SESSION ID: " . substr($session_id_val, -6));
            return $session_id_val;
        } else {
             if ($debug_log) error_log("MC[GetUserID]: Session value found but invalid: " . $_SESSION[$session_key]);
        }
    }

    // 3. Fallback: Could not retrieve ID
    if ($debug_log) error_log("MC[GetUserID]: WARN - Could not retrieve User ID from Cookie or Session.");
    return null; // Return null if no ID found
}

// --- Database Helper Functions ---

/**
 * Retrieves chat history for a given user ID.
 *
 * @param string $persistent_user_id The user's unique ID.
 * @param int $limit_pairs The number of user/assistant message pairs to retrieve.
 * @return array An array of message objects (role, content), oldest first, or empty array.
 */
function minichat_get_db_history($persistent_user_id, $limit_pairs = 10) {
    global $wpdb;
    $tbl = $wpdb->prefix . 'minichat_history';
    $limit = max(0, intval($limit_pairs)) * 2; // Ensure limit is non-negative, get individual messages
    $debug_log = defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG;
    $db_ready_option = defined('MINICHAT_DB_READY_OPTION') ? MINICHAT_DB_READY_OPTION : 'minichat_db_ready_status_v1_6_2';


    if (empty($persistent_user_id)) {
        if ($debug_log) error_log("MC[DB Get]: ERR Empty User ID provided.");
        return [];
    }
    if ($limit <= 0) {
         if ($debug_log) error_log("MC[DB Get]: WARN History limit is 0 or less, skipping DB query for U: " . substr($persistent_user_id, -6));
         return []; // No need to query if limit is 0
    }

    // Check if table exists (essential for DB operations)
    if ($wpdb->get_var($wpdb->prepare("SHOW TABLES LIKE %s", $tbl)) !== $tbl) {
        if ($debug_log) error_log("MC[DB Get]: ERR Table {$tbl} missing U:" . substr($persistent_user_id, -6));
        update_option($db_ready_option, 'no'); // Mark DB as not ready
        return [];
    }

    // Fetch history
    $results = $wpdb->get_results($wpdb->prepare(
        "SELECT role, content FROM {$tbl} WHERE user_id = %s ORDER BY id DESC LIMIT %d",
        $persistent_user_id,
        $limit
    ), ARRAY_A);

    if ($wpdb->last_error) {
        if ($debug_log) error_log("MC[DB Get]: ERR U:" . substr($persistent_user_id, -6) . ": " . $wpdb->last_error);
        return []; // Return empty on error
    }

    return $results ? array_reverse($results) : []; // Reverse to get chronological order (oldest first)
}

/**
 * Saves a chat message to the database.
 *
 * @param string $persistent_user_id User's unique ID.
 * @param string $session_id Current session ID.
 * @param string $role 'user' or 'assistant'.
 * @param string $content The message content.
 * @return int|false The ID of the inserted row or false on failure.
 */
function minichat_save_db_message($persistent_user_id, $session_id, $role, $content) {
    global $wpdb;
    $tbl = $wpdb->prefix . 'minichat_history';
    $debug_log = defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG;
    $db_ready_option = defined('MINICHAT_DB_READY_OPTION') ? MINICHAT_DB_READY_OPTION : 'minichat_db_ready_status_v1_6_2';

    // Basic validation
    if (empty($persistent_user_id) || empty($session_id) || empty($role) || !isset($content)) {
        if ($debug_log) error_log("MC[DB Save]: ERR Missing data U:" . substr($persistent_user_id ?? 'N/A', -6) . " S:" . substr($session_id ?? 'N/A', -6) . " R:" . $role);
        return false;
    }
    // Validate role
    if (!in_array($role, ['user', 'assistant'])) {
         if ($debug_log) error_log("MC[DB Save]: ERR Invalid role '{$role}' U:" . substr($persistent_user_id, -6));
         return false;
    }

    // Check table existence
    if ($wpdb->get_var($wpdb->prepare("SHOW TABLES LIKE %s", $tbl)) !== $tbl) {
        if ($debug_log) error_log("MC[DB Save]: ERR Table {$tbl} missing U:" . substr($persistent_user_id, -6));
        update_option($db_ready_option, 'no');
        return false;
    }

    // Insert data
    $result = $wpdb->insert(
        $tbl,
        [
            'user_id' => $persistent_user_id,
            'session_id' => $session_id,
            'role' => $role,
            'content' => $content, // Content should already be sanitized/prepared before calling this
            'timestamp' => current_time('mysql', 1) // Use WP's current time in GMT
        ],
        ['%s', '%s', '%s', '%s', '%s'] // Data formats
    );

    if ($result === false) {
        if ($debug_log) error_log("MC[DB Save]: ERR U:" . substr($persistent_user_id, -6) . ": " . $wpdb->last_error);
        return false;
    }

    return $wpdb->insert_id; // Return the ID of the newly inserted row
}

// --- OpenAI API Helper ---

/**
 * Fetches the list of available chat models from the OpenAI API.
 * Handles caching via transients. Includes robust error checking.
 *
 * @param string $api_key Your OpenAI API key.
 * @return array An array of suitable model IDs (id => id) or an array with an 'error' key.
 */
function minichat_get_openai_models($api_key) {
    $debug_log = defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG;
    $current_version = defined('MINICHAT_VERSION') ? MINICHAT_VERSION : 'N/A';

    if (empty($api_key)) { return ['error' => __('API Key missing', 'minimal-chatbot')]; }
    $transient_key = 'minichat_openai_models_list_v2'; // Unique key for this version/format

    // Try cache first
    $cached_models = get_transient($transient_key);
    if (is_array($cached_models) && !isset($cached_models['error'])) {
         if ($debug_log) error_log("MC[GetModels]: Returning models from transient '{$transient_key}'.");
        return $cached_models;
    }

    if ($debug_log) error_log("MC[GetModels]: Fetching models from OpenAI API (Cache missed or invalid)...");
    $url = 'https://api.openai.com/v1/models';
    $args = [
        'headers' => [
            'Authorization' => 'Bearer ' . $api_key,
            'User-Agent' => 'MinimalChatbot/' . $current_version . '; WordPress/' . get_bloginfo('version')
         ],
        'timeout' => 20 // Reasonable timeout
    ];
    $response = wp_remote_get($url, $args);

    // Handle connection errors
    if (is_wp_error($response)) {
        $error_message = $response->get_error_message();
        if ($debug_log) error_log("MC[GetModels]: API Connection Error: " . $error_message);
        return ['error' => __('API Connection Error:', 'minimal-chatbot') . ' ' . $error_message];
    }

    $body = wp_remote_retrieve_body($response);
    $code = wp_remote_retrieve_response_code($response);
    $data = json_decode($body, true); // Decode as array

    // Handle non-200 HTTP codes
    if ($code !== 200) {
         $error_msg = sprintf(__('API Error (HTTP %d)', 'minimal-chatbot'), $code);
         if (is_array($data) && isset($data['error']['message'])) { $error_msg .= ": " . $data['error']['message']; }
         elseif (!empty($body)) { $error_msg .= ". Response: " . esc_html(substr($body, 0, 100)); }
         if ($debug_log) error_log("MC[GetModels]: {$error_msg} Full Body: " . $body);
         return ['error' => $error_msg];
    }

    // Handle JSON decoding errors or unexpected format
    if (!is_array($data)) {
         if ($debug_log) error_log("MC[GetModels]: API Error: Invalid JSON response received. Body: " . $body);
         return ['error' => __('API Error: Invalid JSON response format.', 'minimal-chatbot')];
    }
     if (!isset($data['data']) || !is_array($data['data'])) {
         if ($debug_log) error_log("MC[GetModels]: API Error: Response missing 'data' array. Response: " . print_r($data, true));
         return ['error' => __('API Error: Response format incorrect (Missing required data).', 'minimal-chatbot')];
    }

    // Process the valid 'data' array
    $models = [];
    $exclude_patterns = ['vision', 'image', 'embed', 'audio', 'whisper', 'tts', 'davinci', 'babbage', 'curie', 'ada', '-instruct', 'code', 'edit', 'search', 'similarity', 'answers', 'classifications', 'moderation'];
    foreach ($data['data'] as $model) {
        if (!is_array($model) || !isset($model['id'])) continue; // Skip invalid model entries
        $model_id = $model['id'];
        $is_excluded = false;
        foreach($exclude_patterns as $pattern) {
            if (stripos($model_id, $pattern) !== false) { $is_excluded = true; break; }
        }
        // Keep models likely suitable for chat/completion
        if (!$is_excluded && (stripos($model_id, 'gpt') !== false || stripos($model_id, 'completion') !== false || stripos($model_id, 'claude') !== false || stripos($model_id, 'gemini') !== false) ) {
             $models[$model_id] = $model_id; // Use ID as both key and value
        }
    }
    ksort($models); // Sort alphabetically

    // Handle case where filtering results in no models
    if (empty($models)) {
        if ($debug_log) error_log("MC[GetModels]: No suitable chat models found after filtering API response.");
        return ['error' => __('No suitable chat models found from API response.', 'minimal-chatbot')];
    }

    // Save valid models to transient
    set_transient($transient_key, $models, 12 * HOUR_IN_SECONDS); // Cache for 12 hours
    if ($debug_log) error_log("MC[GetModels]: OK fetched & cached " . count($models) . " models in transient '{$transient_key}'.");
    return $models; // Return the array of models
}


// --- Context Caching Cron Job Function ---

/**
 * Function executed by the WP Cron job to update the context cache.
 * Fetches content from selected pages and stores it in a transient.
 */
function minichat_run_context_cache_update() {
    $debug_log = defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG;
    $context_hook = defined('MINICHAT_CONTEXT_UPDATE_HOOK') ? MINICHAT_CONTEXT_UPDATE_HOOK : 'minichat_update_context_cache_hook';
    $cache_limit_default = defined('MINICHAT_DEFAULT_CACHE_LIMIT') ? MINICHAT_DEFAULT_CACHE_LIMIT : 5000;
    $transient_name = defined('MINICHAT_CONTEXT_CACHE_TRANSIENT') ? MINICHAT_CONTEXT_CACHE_TRANSIENT : 'minichat_context_cache_v1_11';

    if ($debug_log) error_log("MC[Cron]: Running context cache update (" . $context_hook . ").");

    $selected_pages = get_option('minichat_context_pages', []); // Get selected page IDs
    $cache_limit = intval(get_option('minichat_context_cache_limit', $cache_limit_default)); // Get char limit

    if (!is_array($selected_pages)) $selected_pages = []; // Ensure it's an array

    if (empty($selected_pages)) {
        delete_transient($transient_name); // Delete transient if no pages are selected
        if ($debug_log) error_log("MC[Cron]: No pages selected for context caching. Cleared transient '{$transient_name}'.");
        return;
    }

    $cached_data = [];
    $total_chars = 0;

    foreach ($selected_pages as $page_id) {
        $page_id = absint($page_id);
        if (!$page_id) continue;

        $post = get_post($page_id);

        // Check if post exists, is a page, and is published
        if ($post && $post->post_type === 'page' && $post->post_status === 'publish') {
            $content = $post->post_content;
            // Basic cleanup: remove tags, decode entities, normalize whitespace
            $content = wp_strip_all_tags($content);
            $content = html_entity_decode($content, ENT_QUOTES | ENT_HTML5, 'UTF-8');
            $content = preg_replace('/\s+/', ' ', $content); // Replace multiple whitespace chars with single space
            $content = trim($content);

            if (!empty($content)) {
                // Truncate if content exceeds limit (use mb_ functions for multibyte safety)
                if (mb_strlen($content) > $cache_limit) {
                    $content = mb_substr($content, 0, $cache_limit) . '... [truncated]';
                     if ($debug_log) error_log("MC[Cron]: Truncated content for Page ID {$page_id} to {$cache_limit} chars.");
                }
                $cached_data[$page_id] = $content;
                $total_chars += mb_strlen($content);
                 if ($debug_log) error_log("MC[Cron]: Cached content for Page ID {$page_id} (" . mb_strlen($content) . " chars).");
            } else {
                 if ($debug_log) error_log("MC[Cron]: Page ID {$page_id} ('".get_the_title($page_id)."') has empty content after cleanup.");
            }
        } else {
             if ($debug_log) error_log("MC[Cron]: Could not retrieve valid published page for ID {$page_id}. Skipping.");
        }
    }

    // Save the data to the transient
    if (!empty($cached_data)) {
        // Cache for 1 day + a bit of buffer to avoid race conditions with the daily cron job
        $expiry = DAY_IN_SECONDS + HOUR_IN_SECONDS;
        set_transient($transient_name, $cached_data, $expiry);
        if ($debug_log) error_log("MC[Cron]: Successfully cached content for " . count($cached_data) . " pages. Total chars: " . $total_chars . ". Transient '{$transient_name}' set with expiry {$expiry}s.");
    } else {
        delete_transient($transient_name); // Ensure transient is deleted if no valid data was cached
        if ($debug_log) error_log("MC[Cron]: No valid content found to cache. Cleared transient '{$transient_name}'.");
    }
}

// --- Keyword Extraction Helper ---

/**
 * Extracts simple keywords from a string.
 * Converts to lowercase, removes basic punctuation, splits by space,
 * removes common stopwords and short words.
 *
 * @param string $text Input text.
 * @param int $min_length Minimum length for a word to be considered a keyword.
 * @return array Array of unique keywords found.
 */
function minichat_get_keywords_from_string($text, $min_length = 4) {
    // Lowercase, remove punctuation (basic), split into words
    $text = strtolower($text ?? '');
    // Remove common punctuation, keep letters, numbers, spaces (Unicode safe)
    $text = preg_replace('/[^\p{L}\p{N}\s]/u', '', $text);
    $words = preg_split('/\s+/', $text, -1, PREG_SPLIT_NO_EMPTY); // Split and remove empty elements

    // Define common stop words (English/German example - adjust as needed)
    $stopwords = [
        'the','a','an','is','are','was','were','in','on','at','to','for','of','it','you','he','she','they','we','i','me','my','myself','us','our','ours','ourselves','your','yours','yourself','yourselves','him','his','himself','her','hers','herself','its','itself','them','their','theirs','themselves','what','which','who','whom','this','that','these','those','am','be','been','being','have','has','had','having','do','does','did','doing','but','if','or','because','as','until','while','about','against','between','into','through','during','before','after','above','below','from','up','down','out','off','over','under','again','further','then','once','here','there','when','where','why','how','all','any','both','each','few','more','most','other','some','such','no','nor','not','only','own','same','so','than','too','very','s','t','can','will','just','don','should','now',
        'der','die','das','ein','eine','eines','er','sie','es','wir','ihr','sind','ist','war','waren','im','am','an','auf','zu','für','von','und','oder','aber','nicht','mit','bei','wie','was','wo','wann','wer','wie','wohin','woher','warum','wieso','weshalb','man','kann','muss','soll','darf','hat','haben','hatte','hatten','wird','werden','wurde','wurden','bin','bist','sein','ihm','ihn','ihnen','ihrer','ihres','ihrem','ihren','seinem','seinen','seiner','seines','mich','dich','sich','uns','euch'
    ];

    $keywords = [];
    if (is_array($words)) {
        foreach ($words as $word) {
            $word = trim($word);
            // Check length and if it's not a stopword and not purely numeric
            if (mb_strlen($word) >= $min_length && !in_array($word, $stopwords) && !is_numeric($word)) {
                $keywords[$word] = 1; // Use array keys for uniqueness
            }
        }
    }
    return array_keys($keywords);
}

?>