<?php
// includes/license-handler.php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Ensure constants are available (might be defined in main plugin file)
if (!defined('MINICHAT_LICENSE_SERVER_URL')) {
    define('MINICHAT_LICENSE_SERVER_URL', 'https://lic.robutech.ch/api'); // Fallback if needed
}
if (!defined('MINICHAT_PLUGIN_API_KEY')) {
    define('MINICHAT_PLUGIN_API_KEY', 'YOUR_FALLBACK_PLUGIN_API_KEY'); // Add a fallback or ensure it's always defined before include
}
if (!defined('MINICHAT_LICENSE_OPTIONS_NAME')) {
    define('MINICHAT_LICENSE_OPTIONS_NAME', 'minichat_license_options');
}
if (!defined('MINICHAT_DEBUG_LOG')) {
    define('MINICHAT_DEBUG_LOG', false); // Default to false if not defined
}


/**
 * Normalizes a domain name for license checks.
 * Converts to lowercase and removes 'www.'.
 *
 * @return string Normalized domain name.
 */
function minichat_get_normalized_domain() {
    $domain = home_url(); // Get the site URL
    $domain = preg_replace('#^https?://#', '', $domain); // Remove http(s)://
    $domain = str_replace('www.', '', strtolower($domain)); // Remove www. and lowercase
    $domain = rtrim($domain, '/'); // Remove trailing slash
    // Handle potential paths if WordPress is installed in a subdirectory
    $domain_parts = explode('/', $domain, 2);
    $domain = $domain_parts[0];
    // Consider edge cases like localhost or IP addresses if needed
    return $domain;
}

/**
 * Sends a request to the license server.
 *
 * @param string $endpoint API endpoint (e.g., '/verify', '/activate').
 * @param string $method 'GET' or 'POST'.
 * @param array $data Data for GET (query params) or POST (body).
 * @return array Decoded JSON response array or an array with an 'error' key on failure.
 */
function minichat_send_license_request($endpoint, $method = 'GET', $data = []) {
    $url = MINICHAT_LICENSE_SERVER_URL . $endpoint;
    $plugin_api_key = defined('MINICHAT_PLUGIN_API_KEY') ? MINICHAT_PLUGIN_API_KEY : '';

    if (empty($plugin_api_key)) {
        if (MINICHAT_DEBUG_LOG) error_log("MC[LicReq]: Plugin API Key for license server is missing.");
        return ['error' => __('Plugin configuration error (Missing Plugin API Key). Please contact support.', 'minimal-chatbot')];
    }

    $headers = [
        'Accept' => 'application/json',
        'X-API-Key' => $plugin_api_key, // Secret key for server auth
        'User-Agent' => 'MinimalChatbot/' . (defined('MINICHAT_VERSION') ? MINICHAT_VERSION : '1.11.0') . '; WordPress/' . get_bloginfo('version'), // Identify the plugin
    ];

    $args = [
        'timeout' => 25, // Slightly longer timeout
        'headers' => $headers,
        'sslverify' => true, // IMPORTANT: Verify SSL certificate
    ];

    if ($method === 'POST') {
        $args['method'] = 'POST';
        $args['headers']['Content-Type'] = 'application/json';
        $args['body'] = json_encode($data);
    } else { // GET request
        $args['method'] = 'GET';
        if (!empty($data)) {
            $url = add_query_arg($data, $url); // Add data as query parameters
        }
    }

     if (MINICHAT_DEBUG_LOG) error_log("MC[LicReq]: Sending {$method} request to {$url}");
     // if (MINICHAT_DEBUG_LOG && $method === 'POST') error_log("MC[LicReq]: POST Body: " . json_encode($data)); // Log POST data if debugging needed

    $response = wp_remote_request($url, $args);

    // --- Handle Connection Errors & Grace Period Activation ---
    if (is_wp_error($response)) {
        $error_message = $response->get_error_message();
        if (MINICHAT_DEBUG_LOG) error_log("MC[LicReq]: ERROR - wp_remote_request failed: " . $error_message);

        // Activate Grace Period ONLY if a license key exists and was previously valid
        $license_options = get_option(MINICHAT_LICENSE_OPTIONS_NAME, []);
        if (!empty($license_options['key']) && isset($license_options['valid']) && $license_options['valid'] === true) {
            minichat_activate_grace_period(__('Connection Error:', 'minimal-chatbot') . ' ' . $error_message);
        }
        return ['error' => __('License server connection error:', 'minimal-chatbot') . ' ' . $error_message];
    }

    // --- Handle HTTP Errors & Grace Period Activation ---
    $http_code = wp_remote_retrieve_response_code($response);
    $body = wp_remote_retrieve_body($response);
    $decoded_body = json_decode($body, true); // Decode as array

    if ($http_code >= 400 || json_last_error() !== JSON_ERROR_NONE) {
        $error_message = sprintf(__('License server communication error (HTTP %d).', 'minimal-chatbot'), $http_code);
        if (json_last_error() !== JSON_ERROR_NONE) {
             $error_message .= ' ' . __('Invalid response format received.', 'minimal-chatbot');
        }

        if (isset($decoded_body['message'])) { // Use server's message if available
             $error_message .= " Server said: " . esc_html($decoded_body['message']);
        } elseif (!empty($body) && $http_code < 500) { // Show snippet for client errors if no JSON message
             $error_message .= " Response: " . esc_html(substr($body, 0, 150)) . "...";
        }

        if (MINICHAT_DEBUG_LOG) error_log("MC[LicReq]: ERROR - {$error_message}. Full Body: " . $body);

        // Activate Grace Period ONLY if a license key exists and was previously valid
        $license_options = get_option(MINICHAT_LICENSE_OPTIONS_NAME, []);
        if (!empty($license_options['key']) && isset($license_options['valid']) && $license_options['valid'] === true) {
             minichat_activate_grace_period($error_message);
        }

        return ['error' => $error_message];
    }

    // --- Handle Success & Grace Period Reset ---
    if (MINICHAT_DEBUG_LOG) error_log("MC[LicReq]: SUCCESS - HTTP {$http_code}. Response: " . print_r($decoded_body, true));

    // Reset grace period if it was active and the server is now reachable and responds successfully
    minichat_reset_grace_period();

    // Return the decoded JSON response array
    return is_array($decoded_body) ? $decoded_body : ['error' => __('Invalid response format received.', 'minimal-chatbot')];
}

/**
 * Activates the grace period for license validation.
 *
 * @param string $error_reason The reason for activating grace period.
 * @return bool Whether grace period was activated or already active and valid.
 */
function minichat_activate_grace_period($error_reason = '') {
    $grace_options_key = 'minichat_grace_period';
    $grace_options = get_option($grace_options_key, []);
    $now = time();
    $grace_duration = 2 * DAY_IN_SECONDS; // 2 days grace period

    // Check if already in a valid grace period
    if (isset($grace_options['active']) && $grace_options['active'] === true && isset($grace_options['expires']) && $grace_options['expires'] > $now) {
        if (MINICHAT_DEBUG_LOG) error_log("MC[Grace]: Already in active grace period.");
        return true; // Already active and valid
    }

    // Activate new grace period
    $grace_period = [
        'active' => true,
        'started' => $now,
        'expires' => $now + $grace_duration,
        'reason' => sanitize_text_field($error_reason),
        'last_updated' => current_time('mysql', 1) // Use GMT time
    ];
    update_option($grace_options_key, $grace_period);

    if (MINICHAT_DEBUG_LOG) error_log("MC[Grace]: Activated until " .
        date('Y-m-d H:i:s', $grace_period['expires']) . ". Reason: " . $error_reason);
    return true;
}

/**
 * Resets the grace period if active.
 */
function minichat_reset_grace_period() {
    $grace_options_key = 'minichat_grace_period';
    $grace_options = get_option($grace_options_key, []);

    // Reset only if currently active
    if (isset($grace_options['active']) && $grace_options['active'] === true) {
        // Keep the record but mark as inactive
        $grace_options['active'] = false;
        $grace_options['last_updated'] = current_time('mysql', 1); // Use GMT time
        $grace_options['reset_reason'] = 'Server connection restored';
        update_option($grace_options_key, $grace_options);

        if (MINICHAT_DEBUG_LOG) error_log("MC[Grace]: Reset - server connection restored or license validated.");
    }
}

/**
 * Checks if grace period is active and valid.
 *
 * @return bool True if grace period is active and not expired.
 */
function minichat_is_grace_period_active() {
    $grace_options = get_option('minichat_grace_period', []);
    $now = time();

    if (!empty($grace_options) && isset($grace_options['active']) && $grace_options['active'] === true &&
        isset($grace_options['expires']) && $grace_options['expires'] > $now) {
        return true;
    }

    return false;
}

/**
 * Get grace period status information.
 *
 * @return array Grace period status information.
 */
function minichat_get_grace_period_info() {
    $grace_options = get_option('minichat_grace_period', []);
    $now = time();
    $default_info = [
        'active' => false,
        'time_remaining' => 0,
        'time_remaining_human' => __('Not active', 'minimal-chatbot'),
        'reason' => '',
        'started' => '',
        'expires' => ''
    ];

    if (empty($grace_options) || !isset($grace_options['expires'])) {
        return $default_info;
    }

    $is_active_now = isset($grace_options['active']) && $grace_options['active'] === true && $grace_options['expires'] > $now;
    $time_remaining = $is_active_now ? max(0, $grace_options['expires'] - $now) : 0;

    // Format time remaining
    if ($is_active_now) {
        $days = floor($time_remaining / DAY_IN_SECONDS);
        $hours = floor(($time_remaining % DAY_IN_SECONDS) / HOUR_IN_SECONDS);
        $minutes = floor(($time_remaining % HOUR_IN_SECONDS) / MINUTE_IN_SECONDS);

        $time_human = '';
        if ($days > 0) $time_human .= $days . ' ' . _n('day', 'days', $days, 'minimal-chatbot') . ', ';
        if ($hours > 0) $time_human .= $hours . ' ' . _n('hour', 'hours', $hours, 'minimal-chatbot') . ', ';
        $time_human .= $minutes . ' ' . _n('minute', 'minutes', $minutes, 'minimal-chatbot');
    } else {
        $time_human = __('Expired or Inactive', 'minimal-chatbot');
    }


    $started_ts = $grace_options['started'] ?? 0;
    $expires_ts = $grace_options['expires'] ?? 0;

    return [
        'active' => $is_active_now,
        'time_remaining' => $time_remaining,
        'time_remaining_human' => $time_human,
        'reason' => $grace_options['reason'] ?? '',
        'started' => $started_ts ? date_i18n(get_option('date_format') . ' H:i:s', $started_ts) : '',
        'expires' => $expires_ts ? date_i18n(get_option('date_format') . ' H:i:s', $expires_ts) : ''
    ];
}

// --- Trial Period Functions ---

/**
 * Activates a trial period for the chatbot.
 *
 * @return array Trial status information or error array.
 */
function minichat_activate_trial() {
    $trial_options_key = 'minichat_trial_options';
    $trial_options = get_option($trial_options_key, []);
    $now = time();
    $trial_duration = 14 * DAY_IN_SECONDS; // 14 days trial

    // Check if trial was already used
    if (!empty($trial_options) && isset($trial_options['trial_used']) && $trial_options['trial_used'] === true) {
        if (MINICHAT_DEBUG_LOG) error_log("MC[Trial]: Attempt to activate already used trial.");
        return [
            'success' => false,
            'message' => __('Trial period has already been used on this site.', 'minimal-chatbot')
        ];
    }
     // Check if a full license is already active
    $license_options = get_option(MINICHAT_LICENSE_OPTIONS_NAME, []);
    if (isset($license_options['valid']) && $license_options['valid'] === true) {
         if (MINICHAT_DEBUG_LOG) error_log("MC[Trial]: Attempt to activate trial while full license is active.");
         return [
            'success' => false,
            'message' => __('A full license is already active. Trial cannot be started.', 'minimal-chatbot')
        ];
    }

    // Define trial period
    $trial_period = [
        'active' => true,
        'trial_used' => true, // Mark trial as used permanently
        'started' => $now,
        'expires' => $now + $trial_duration,
        'last_updated' => current_time('mysql', 1) // GMT time
    ];
    update_option($trial_options_key, $trial_period);

    if (MINICHAT_DEBUG_LOG) error_log("MC[Trial]: Activated until " .
        date('Y-m-d H:i:s', $trial_period['expires']));
    return [
        'success' => true,
        'message' => __('Trial period activated successfully!', 'minimal-chatbot'),
        'trial' => $trial_period // Return trial details
    ];
}

/**
 * Checks if trial period is active and valid.
 *
 * @return bool True if trial is active and not expired.
 */
function minichat_is_trial_active() {
    $trial_options = get_option('minichat_trial_options', []);
    $now = time();

    if (!empty($trial_options) && isset($trial_options['active']) && $trial_options['active'] === true &&
        isset($trial_options['expires']) && $trial_options['expires'] > $now) {
        return true;
    }

    return false;
}

/**
 * Get trial status information.
 *
 * @return array Trial status information.
 */
function minichat_get_trial_info() {
    $trial_options = get_option('minichat_trial_options', []);
    $now = time();
    $default_info = [
        'active' => false,
        'trial_used' => false,
        'time_remaining' => 0,
        'time_remaining_human' => __('Not started', 'minimal-chatbot'),
        'days_remaining' => 0,
        'started' => '',
        'expires' => ''
    ];

    if (empty($trial_options)) {
        return $default_info;
    }

    $is_active_now = isset($trial_options['active']) && $trial_options['active'] === true &&
                     isset($trial_options['expires']) && $trial_options['expires'] > $now;

    $time_remaining = $is_active_now ? max(0, $trial_options['expires'] - $now) : 0;
    $days_remaining = $is_active_now ? ceil($time_remaining / DAY_IN_SECONDS) : 0;

     // Format time remaining
    if ($is_active_now) {
        $time_human = $days_remaining . ' ' . _n('day', 'days', $days_remaining, 'minimal-chatbot');
    } else {
        $time_human = ($trial_options['trial_used'] ?? false) ? __('Expired or Inactive', 'minimal-chatbot') : __('Not started', 'minimal-chatbot');
    }

    $started_ts = $trial_options['started'] ?? 0;
    $expires_ts = $trial_options['expires'] ?? 0;

    return [
        'active' => $is_active_now,
        'trial_used' => $trial_options['trial_used'] ?? false,
        'time_remaining' => $time_remaining,
        'time_remaining_human' => $time_human,
        'days_remaining' => $days_remaining,
        'started' => $started_ts ? date_i18n(get_option('date_format'), $started_ts) : '', // Show only date started
        'expires' => $expires_ts ? date_i18n(get_option('date_format'), $expires_ts) : '' // Show only date expires
    ];
}

/**
 * Verifies a license key with the remote server and updates options.
 *
 * @param string $license_key The license key to verify.
 * @return array The result array from the server or an array with 'error' key.
 */
function minichat_verify_license_remote($license_key) {
    if (empty($license_key)) {
        return ['error' => __('License key required for verification.', 'minimal-chatbot')];
    }

    $domain = minichat_get_normalized_domain();
    $data = [
        'license_key' => $license_key,
        'domain' => $domain,
    ];
    $result = minichat_send_license_request('/verify', 'GET', $data);

    // Prepare options to save, based on result
    $current_options = get_option(MINICHAT_LICENSE_OPTIONS_NAME, []); // Get current options to preserve key if verify fails
    $options_to_save = [
        'key' => $license_key, // Always save the key being checked
        'status' => 'error',
        'valid' => false,
        'message' => __('Could not connect or verify license.', 'minimal-chatbot'),
        'last_checked' => current_time('mysql', 1), // GMT time
        'data' => null, // Reset license data on failed verification
    ];

    if (isset($result['error'])) {
        $options_to_save['message'] = $result['error'];
        // Keep existing license data if verification failed due to connection issue? Maybe not.
        $options_to_save['data'] = $current_options['data'] ?? null; // Preserve old data on connection error only? Let's clear it.
    } else {
        // Map server response to our options format
        $options_to_save['status'] = isset($result['status']) ? sanitize_key($result['status']) : 'unknown';
        $options_to_save['valid'] = isset($result['valid']) ? (bool)$result['valid'] : false;
        $options_to_save['message'] = isset($result['message']) ? sanitize_text_field($result['message']) : __('Verification completed.', 'minimal-chatbot');
        $options_to_save['data'] = isset($result['license']) ? $result['license'] : null; // Store license details if provided
    }

    // Save the updated status to WordPress options
    update_option(MINICHAT_LICENSE_OPTIONS_NAME, $options_to_save);

    if (MINICHAT_DEBUG_LOG) error_log("MC[LicVerify]: Updated license options after verify. Status: " . $options_to_save['status'] . ", Valid: " . ($options_to_save['valid'] ? 'Yes' : 'No'));

    // Return the original server result (or error array) to the caller
    return $result;
}


/**
 * Activates a license key with the remote server and updates options.
 * **Also deletes trial options on successful activation.**
 *
 * @param string $license_key The license key to activate.
 * @return array The result array from the server or an array with 'error' key.
 */
function minichat_activate_license_remote($license_key) {
     if (empty($license_key)) {
        return ['error' => __('License key required for activation.', 'minimal-chatbot')];
    }

    $domain = minichat_get_normalized_domain();
    $data = [
        'license_key' => $license_key,
        'domain' => $domain,
    ];
    $result = minichat_send_license_request('/activate', 'POST', $data);

    // Prepare options to save, based on result
    $options_to_save = [
        'key' => $license_key, // Always save the key being activated
        'status' => 'error',
        'valid' => false,
        'message' => __('Could not activate license.', 'minimal-chatbot'),
        'last_checked' => current_time('mysql', 1), // Record check time on activation attempt too
        'data' => null,
    ];

    if (isset($result['error'])) {
        $options_to_save['message'] = $result['error'];
    } else {
        // Map server response to our options format
        $options_to_save['status'] = isset($result['status']) ? sanitize_key($result['status']) : 'unknown';
        // Activation is successful if server explicitly says 'valid' is true OR status is 'activated'/'already_active'
        $options_to_save['valid'] = (isset($result['valid']) && $result['valid'] === true) ||
                                    in_array($options_to_save['status'], ['activated', 'already_active']);
        $options_to_save['message'] = isset($result['message']) ? sanitize_text_field($result['message']) : __('Activation request processed.', 'minimal-chatbot');
        $options_to_save['data'] = isset($result['license']) ? $result['license'] : null;
    }

    // Save the updated status to WordPress options
    update_option(MINICHAT_LICENSE_OPTIONS_NAME, $options_to_save);

    if (MINICHAT_DEBUG_LOG) error_log("MC[LicActivate]: Updated license options after activate. Status: " . $options_to_save['status'] . ", Valid: " . ($options_to_save['valid'] ? 'Yes' : 'No'));

    // --- !!! TRIAL BUGFIX !!! ---
    // If activation was successful and the license is now valid, delete the trial options.
    if ($options_to_save['valid'] === true) {
        $trial_options_key = 'minichat_trial_options';
        if (get_option($trial_options_key) !== false) { // Check if option exists before deleting
            delete_option($trial_options_key);
            if (MINICHAT_DEBUG_LOG) error_log("MC[LicActivate]: Successfully activated license. Deleted trial options ('{$trial_options_key}').");
        }
    }
    // --- END TRIAL BUGFIX ---

    // Return the original server result (or error array)
    return $result;
}

/**
 * Checks if the chatbot should be operational (Valid License OR Active Trial OR Active Grace Period).
 * This is the central function to check before enabling core functionality (AJAX, Enqueue).
 *
 * @return bool True if license is valid OR grace period is active OR trial is active.
 */
function minichat_is_license_valid() {
    // 1. Check for a valid license stored locally
    $license_options = get_option(MINICHAT_LICENSE_OPTIONS_NAME, []);
    if (isset($license_options['valid']) && $license_options['valid'] === true) {
        // Optional: Add a check for expiry date from $license_options['data']['expiry_date'] if relevant
        // $expiry_ts = isset($license_options['data']['expiry_date']) ? strtotime($license_options['data']['expiry_date']) : 0;
        // if ($expiry_ts === 0 || $expiry_ts > time()) { // Check if expiry is not set or in the future
             return true; // Valid license found
        // }
    }

    // 2. Check for an active grace period
    if (minichat_is_grace_period_active()) {
         if (MINICHAT_DEBUG_LOG) error_log("MC[LicCheck]: License invalid/missing, but Grace Period is active.");
        return true; // Grace period active
    }

    // 3. Check for an active trial period
    if (minichat_is_trial_active()) {
         if (MINICHAT_DEBUG_LOG) error_log("MC[LicCheck]: License invalid/missing, but Trial Period is active.");
        return true; // Trial period active
    }

    // 4. If none of the above, license is not valid/active
    if (MINICHAT_DEBUG_LOG) error_log("MC[LicCheck]: No valid license, grace period, or trial period found.");
    return false;
}

// Helper function for sanitizing license options before saving (if needed, e.g. from bulk import/manual set)
// Kept identical to previous versions, assuming it's used correctly elsewhere or by WordPress directly.
function minichat_sanitize_license_options($input){
    $output = [];
    $output['key'] = isset($input['key']) ? sanitize_text_field($input['key']) : '';
    $output['status'] = isset($input['status']) ? sanitize_key($input['status']) : 'inactive';
    $output['valid'] = isset($input['valid']) ? (bool)$input['valid'] : false;
    $output['message'] = isset($input['message']) ? sanitize_text_field($input['message']) : '';
    // Data can be complex (e.g., nested array from API), avoid aggressive sanitization that breaks it.
    // Basic check to ensure it's an array might be good if structure is known.
    $output['data'] = isset($input['data']) ? $input['data'] : null;
    $output['last_checked'] = isset($input['last_checked']) ? sanitize_text_field($input['last_checked']) : '';
    return $output;
}

?>