<?php
/**
 * WooCommerce Integration Logic for Minimal Chatbot Add-on
 *
 * Stellt die Funktion bereit, die als Werkzeug (Tool) von der KI über den
 * AJAX-Handler aufgerufen wird, um WooCommerce-Produkte zu suchen.
 * Version: 2.0.0 (Tool Calling Refactoring)
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Die alten Filter-Hooks und -Funktionen sind nicht mehr nötig und wurden entfernt.
// Die alte globale Variable $minichat_wc_request_product_data wird nicht mehr benötigt.

/**
 * Führt die WooCommerce-Produktsuche als "Werkzeug" aus, basierend auf Argumenten von der KI.
 *
 * Diese Funktion wird vom AJAX-Handler aufgerufen, wenn die KI das Werkzeug 'produktSucheImShop' anfordert.
 *
 * @param array $arguments Ein assoziatives Array mit Suchargumenten, die von der KI bestimmt wurden.
 * Erwartete Schlüssel (alle optional): 'suchbegriff', 'kategorie_slug', 'alle_auflisten'.
 * Beispiel: ['suchbegriff' => 'T-Shirt', 'kategorie_slug' => 'bekleidung']
 * Beispiel: ['alle_auflisten' => true]
 *
 * @return array Ein Array mit zwei Schlüsseln:
 * 'text_for_ai' => (string) Eine formatierte Textzusammenfassung der Ergebnisse für die KI.
 * 'structured_data' => (array) Ein Array mit strukturierten Produktdaten für das Frontend (Produktkarten). Leer, wenn nichts gefunden wurde.
 */
function minichat_wc_tool_execute_search($arguments = []) {
    if (!function_exists('wc_get_products')) {
        return [
            'text_for_ai'     => 'Fehler: WooCommerce scheint nicht korrekt zu funktionieren (wc_get_products fehlt).',
            'structured_data' => []
        ];
    }

    // Standard-Argumente für wc_get_products
    $args = [
        'status' => 'publish',
        'limit'  => intval(get_option('minichat_wc_max_products', 5)), // Limit aus den Add-on Einstellungen
        'paginate' => false,
        'return' => 'objects',
    ];

    // Verarbeite Argumente von der KI sicher
    $search_term   = isset($arguments['suchbegriff']) ? sanitize_text_field($arguments['suchbegriff']) : '';
    $category_slug = isset($arguments['kategorie_slug']) ? sanitize_key($arguments['kategorie_slug']) : '';
    $list_all      = isset($arguments['alle_auflisten']) ? (bool)$arguments['alle_auflisten'] : false;
    // Hier könnten weitere Argumente (Preis, Attribute etc.) verarbeitet werden, wenn das Tool in der KI-Definition erweitert wird.

    // --- Baue die Query-Argumente basierend auf KI-Parametern ---

    // Füge Suchbegriff hinzu, AUSSER wenn 'alle_auflisten' gewünscht ist.
    if (!$list_all && !empty($search_term)) {
        $args['s'] = $search_term;
    }

    // Füge Kategorie-Filter hinzu
    if (!empty($category_slug)) {
        $term = get_term_by('slug', $category_slug, 'product_cat');
        if ($term instanceof WP_Term) {
            if (!isset($args['tax_query'])) { $args['tax_query'] = []; } // Initialisiere tax_query falls nötig
            $args['tax_query'][] = [
                'taxonomy' => 'product_cat',
                'field'    => 'term_id',
                'terms'    => $term->term_id,
                'operator' => 'IN'
            ];
            // Relation hinzufügen, falls später mehr Taxonomien dazukommen
            if(count($args['tax_query']) > 1 && !isset($args['tax_query']['relation'])) {
                 $args['tax_query']['relation'] = 'AND';
            }
        } else {
             if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) { error_log("MC WC Tool: Von KI angeforderte Kategorie '{$category_slug}' nicht gefunden."); }
             // Informiere die KI, dass die Kategorie nicht gefunden wurde? Optional.
        }
    }

    // Hier könnten weitere Filter (Attribute, Preis) basierend auf KI-Argumenten hinzugefügt werden...

    // --- Führe die Suche aus ---
    if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) { error_log("MC WC Tool: Calling wc_get_products with args: " . print_r($args, true)); }
    $products = wc_get_products($args);
    if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) { error_log("MC WC Tool: wc_get_products returned " . count($products) . " products."); }


    // --- Formatiere das Ergebnis ---
    $structured_results = [];
    $result_string_for_ai = '';

    if (empty($products)) {
        // Baue "Nichts gefunden"-Nachricht für KI
        if (!empty($search_term)) {
            $result_string_for_ai = sprintf('Es wurden keine Produkte für den Suchbegriff "%s" gefunden.', esc_html($search_term));
        } elseif (!empty($category_slug)) {
             $result_string_for_ai = sprintf('Es wurden keine Produkte in der Kategorie "%s" gefunden.', esc_html($category_slug));
        } elseif ($list_all) {
            $result_string_for_ai = 'Es wurden keine Produkte im Shop gefunden (vielleicht ist er leer?).';
        } else {
             $result_string_for_ai = 'Keine passenden Produkte für die angegebenen Kriterien gefunden.';
        }
         if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) { error_log("MC WC Tool: No products found. AI text: " . $result_string_for_ai); }

    } else {
        // Baue Ergebnis-String für KI und strukturiertes Array fürs Frontend
        $result_string_for_ai = "Folgende Produkte wurden gefunden (maximal " . $args['limit'] . "):\n";
        $wcs_active = class_exists('WC_Subscriptions') && function_exists('wcs_is_product_subscription');
        $image_size_setting = get_option('minichat_wc_image_size', 'woocommerce_thumbnail');
        $include_short_desc_setting = get_option('minichat_wc_show_short_desc', false);

        foreach ($products as $product) {
             if (!$product instanceof WC_Product) continue;

             $product_id = $product->get_id();
             $product_name = $product->get_name();
             $product_permalink = $product->get_permalink();
             $product_image_id = $product->get_image_id();
             $product_image_url = $product_image_id ? wp_get_attachment_image_url($product_image_id, $image_size_setting) : wc_placeholder_img_src($image_size_setting);
             $product_stock_status = $product->get_stock_status();
             $product_stock_quantity = $product->managing_stock() ? $product->get_stock_quantity() : null;

             // Detaillierten Preis-String für KI holen
             $price_string_for_ai = _minichat_wc_get_detailed_price_string($product, $wcs_active);

             // Stock Info für KI
             $stock_info_for_ai = $product_stock_status;
             if ($product_stock_status === 'instock' && $product_stock_quantity !== null) {
                 $stock_info_for_ai .= ' (' . $product_stock_quantity . ' verfügbar)';
             }

             // String für KI zusammenbauen
             $result_string_for_ai .= sprintf("- %s (ID: %d, Preis: %s, Status: %s)\n",
                 $product_name,
                 $product_id,
                 $price_string_for_ai, // Detaillierter Preis
                 $stock_info_for_ai
             );

             // Strukturiertes Array fürs Frontend bauen
             $product_short_description = $include_short_desc_setting ? wp_strip_all_tags($product->get_short_description()) : '';
             // Hole price_html für die Frontend-Darstellung (mit HTML)
             $price_html_for_frontend = $product->get_price_html();
              if (empty($price_html_for_frontend)) {
                 if ($product->is_type('variable') || $product->is_type('grouped')) { $price_html_for_frontend = __('Price varies', 'wc-minichat-addon'); }
                 else { if (method_exists($product, 'get_price') && $product->get_price()) { $price_html_for_frontend = wc_price($product->get_price()); } else { $price_html_for_frontend = __('Price not available', 'wc-minichat-addon'); } }
             }

             $structured_results[] = [
                'id' => $product_id,
                'title' => $product_name,
                'price_html' => $price_html_for_frontend, // HTML für Frontend
                'url' => $product_permalink,
                'image_url' => $product_image_url ?: '',
                'stock_status' => $product_stock_status,
                'stock_quantity' => $product_stock_quantity,
                'short_description' => $product_short_description,
            ];
        }
         if (defined('MINICHAT_DEBUG_LOG') && MINICHAT_DEBUG_LOG) { error_log("MC WC Tool: Formatted " . count($structured_results) . " products. AI text length: " . strlen($result_string_for_ai)); }
    }

    // Gib BEIDES zurück: Text für KI und Daten fürs Frontend
    return [
        'text_for_ai'     => $result_string_for_ai,
        'structured_data' => $structured_results
    ];
}

/**
 * Interne Hilfsfunktion zur Erstellung eines detaillierten Preis-Strings für die KI,
 * insbesondere für Abo-Produkte.
 *
 * @param WC_Product $product Das WooCommerce Produktobjekt.
 * @param bool $wcs_active Ist das WooCommerce Subscriptions Plugin aktiv?
 * @return string Ein formatierter Preis-String.
 */
function _minichat_wc_get_detailed_price_string(WC_Product $product, $wcs_active) {
    $price_string = '';

    // Prüfe auf Abo-Produkt
    if ($wcs_active && function_exists('wcs_is_product_subscription') && wcs_is_product_subscription($product)) {
        // Versuche, detaillierte Abo-Infos zu bekommen
        $price = WC_Subscriptions_Product::get_price($product);
        $period = WC_Subscriptions_Product::get_period($product); // 'day', 'week', 'month', 'year'
        $interval = WC_Subscriptions_Product::get_interval($product); // e.g., 1, 2, 3...

        if ($price !== '' && $period && $interval) {
            // Baue String wie "12.00 CHF pro 1 Monat" oder "120.00 CHF pro 1 Jahr"
            $price_string = wc_price($price) . ' pro';
            if ($interval > 1) {
                $price_string .= ' ' . $interval;
            }
            // Übersetzungen für Perioden (vereinfacht)
            $period_translation = [ 'day' => 'Tag', 'week' => 'Woche', 'month' => 'Monat', 'year' => 'Jahr' ];
            $translated_period = isset($period_translation[$period]) ? $period_translation[$period] : $period;
            // Pluralisierung (sehr einfach)
            if ($interval > 1 && $translated_period !== 'Woche') { // Wochen bleibt Woche
                 $translated_period .= ($translated_period === 'Tag' || $translated_period === 'Monat' || $translated_period === 'Jahr') ? 'e' : '';
                 if ($translated_period === 'Jahre') $translated_period = 'Jahr'; // Ausnahme? Oder Plural? Nimm Singular.
                 if ($translated_period === 'Monate') $translated_period = 'Monat'; // Nimm Singular für "pro X ..."
                 if ($translated_period === 'Tage') $translated_period = 'Tag';
                 // Besser: WordPress Internationalization _n() nutzen, wenn möglich
            }
             $price_string .= ' ' . $translated_period;

             // Füge ggf. Anmeldegebühr hinzu
             $sign_up_fee = WC_Subscriptions_Product::get_sign_up_fee($product);
             if ($sign_up_fee > 0) {
                 $price_string .= ' (zzgl. ' . wc_price($sign_up_fee) . ' Anmeldegebühr)';
             }

        } else {
            // Fallback: Nutze price_html und entferne HTML
            $price_html = $product->get_price_html();
            $price_string = !empty($price_html) ? strip_tags($price_html) : 'Preis nicht verfügbar';
        }
    }
    // Für variable Produkte (die auch Abos sein können!)
    elseif ($product->is_type('variable') || $product->is_type('variable-subscription')) {
         // Preisspanne für variable Produkte holen
         $min_price = $product->get_variation_price('min', true);
         $max_price = $product->get_variation_price('max', true);
         if ($min_price !== $max_price) {
             // Spezifische Formatierung für Abo-Preisspannen holen, falls möglich
              if ($wcs_active && function_exists('wcs_get_price_string') && $product->is_type('variable-subscription')) {
                 // Versuche, die komplexere Abo-Preis-Spanne zu bekommen
                  $price_string = wcs_get_price_string($product, ['price' => '%price%']); // Einfache Version
                  // Entferne HTML robust
                  $price_string = preg_replace('/<[^>]*>/', ' ', $price_string); // Ersetze Tags durch Leerzeichen
                  $price_string = preg_replace('/\s+/', ' ', $price_string); // Normalisiere Leerzeichen
                  $price_string = trim($price_string);
             } else {
                  // Normale Preisspanne
                 $price_string = wc_price($min_price) . ' – ' . wc_price($max_price);
             }
         } else {
              // Nur ein Preis für alle Variationen (oder kein Preis)
              $price_string = $min_price ? wc_price($min_price) : 'Preis nicht verfügbar';
               // Wenn die einzelne Variation ein Abo ist... komplexer Fall
               // Hier könnte man die erste verfügbare Variation prüfen
         }
    }
    // Für andere Produkttypen
    else {
        // Normale Preis-Logik
        $price_html = $product->get_price_html();
        $price_string = !empty($price_html) ? strip_tags($price_html) : ($product->get_price() ? wc_price($product->get_price()) : 'Preis nicht verfügbar');
    }

    // Bereinige doppelte Leerzeichen etc.
    $price_string = preg_replace('/\s+/', ' ', $price_string);
    return trim($price_string);
}

?>