/**
 * Minimal Chatbot Frontend JavaScript
 * Version: 1.12.1 (Mit Event für WC Addon)
 */
jQuery(document).ready(function($) {

    // --- Check Prereqs ---
    const container = $('#minichat-container');
    // Check for core data structure more reliably
    if (!container.length || typeof minichat_data === 'undefined' || typeof minichat_data.ajax_url === 'undefined' || typeof minichat_data.settings === 'undefined') {
        console.error("Minimal Chatbot: Container or essential minichat_data missing, aborting JS.");
        return;
    }

    // --- References ---
    const widget = $('#minichat-widget');
    const header = $('#minichat-header');
    const body = $('#minichat-body');
    const messagesContainer = $('#minichat-messages');
    const inputField = $('#minichat-input');
    const sendButton = $('#minichat-send-button');
    const voiceButton = $('#minichat-voice-button'); // May not exist if STT disabled
    const minimizeBtn = header.find('.minichat-minimize-btn');
    const settingsBtn = header.find('.minichat-settings-btn');
    const optionsPopup = $('#minichat-options-popup');
    const clearChatBtn = $('#minichat-clear-chat-btn');
    const settings = minichat_data.settings || {}; // Use empty object as fallback
    const ajax_url = minichat_data.ajax_url;
    const nonces = { // Store nonces for easier access
        message: minichat_data.nonce,
        feedback: minichat_data.feedback_nonce,
        clear: minichat_data.clear_chat_nonce,
        // license and context nonces are used in admin, not typically here
    };

    // Simple check for debug mode potentially passed from PHP
    const debugEnabled = settings.enable_debug === true; // Check specific setting if passed

    // --- State ---
    let isMinimized = widget.hasClass('minimized');
    let isRecording = false;
    let speechRecognition = null;
    let synthesis = window.speechSynthesis;
    let currentUtterance = null;
    let inactivityTimer = null;
    const minimizedHeight = "50px"; // Define minimized height
    const maximizedHeight = "500px"; // Define maximized height (adjust based on CSS or desired fixed height)

    // --- Initialization ---
    function initializeFeatures() {
        // Init Speech Recognition (STT)
        if (settings.enable_stt && ('webkitSpeechRecognition' in window || 'SpeechRecognition' in window)) {
            const SpeechRecognition = window.SpeechRecognition || window.webkitSpeechRecognition;
            speechRecognition = new SpeechRecognition();
            speechRecognition.continuous = false;
            speechRecognition.interimResults = false;
            try {
                // Use site language if available, otherwise browser default or fallback
                const siteLang = $('html').attr('lang') || 'de-DE'; // Example fallback
                speechRecognition.lang = siteLang;
            } catch(e) {
                if(debugEnabled) console.warn("MC JS: Could not set STT lang", e);
            }

            speechRecognition.onstart = function() {
                isRecording = true;
                if(voiceButton.length) voiceButton.addClass('recording').attr('title', 'Aufnahme stoppen');
                inputField.attr('placeholder', 'Höre zu...');
                inputField.removeClass('error'); // Clear previous error state
            };
            speechRecognition.onresult = function(event) {
                if (event.results && event.results[0] && event.results[0][0]) {
                    inputField.val(event.results[0][0].transcript);
                    // Optional: Automatically send after successful recognition?
                    // sendMessage();
                }
            };
            speechRecognition.onerror = function(event) {
                console.error('STT Error:', event.error, event.message);
                let msg = 'Sprachfehler.'; // Generic error
                if (event.error === 'no-speech') {
                    msg = 'Nichts gehört.';
                } else if (event.error === 'audio-capture') {
                    msg = 'Mikrofonproblem?';
                } else if (event.error === 'not-allowed') {
                    msg = 'Mikrofon blockiert.';
                    // Maybe disable the button permanently if blocked?
                } else if (event.error === 'network') {
                     msg = 'Netzwerkfehler.';
                }
                inputField.attr('placeholder', msg).addClass('error');
                setTimeout(() => {
                    if (inputField.attr('placeholder') === msg) { // Only reset if it wasn't changed manually
                       inputField.attr('placeholder', 'Schreibe eine Nachricht...').removeClass('error');
                    }
                }, 3000);
                 // Ensure recording state is reset
                 isRecording = false;
                 if(voiceButton.length) voiceButton.removeClass('recording').attr('title', 'Spracheingabe');
            };
            speechRecognition.onend = function() {
                // This runs even after errors sometimes, so double-check state
                 isRecording = false;
                 if(voiceButton.length) voiceButton.removeClass('recording').attr('title', 'Spracheingabe');
                 // Reset placeholder if it was 'Listening...' or in error state related to listening
                 const currentPlaceholder = inputField.attr('placeholder');
                 if(currentPlaceholder === 'Höre zu...' || inputField.hasClass('error')) {
                    inputField.attr('placeholder','Schreibe eine Nachricht...').removeClass('error');
                 }
                 // Refocus input field if widget is open
                 if (!isMinimized) {
                     inputField.focus();
                 }
            };

            // Attach click handler to voice button
            if (voiceButton.length && speechRecognition) {
                voiceButton.on('click', function() {
                    if (!speechRecognition) return;
                    if (isRecording) {
                        speechRecognition.stop();
                    } else {
                        // Check permissions before starting (modern browsers)
                        if (navigator.permissions && typeof navigator.permissions.query === 'function') {
                            navigator.permissions.query({ name: 'microphone' })
                                .then(function(permissionStatus) {
                                    if (permissionStatus.state === 'denied') {
                                        alert("Der Zugriff auf das Mikrofon wurde blockiert. Bitte überprüfe die Browsereinstellungen.");
                                        return;
                                    }
                                    // If granted or prompt, proceed to start
                                    try {
                                        speechRecognition.start();
                                    } catch(e) {
                                        console.error("Error starting STT:", e);
                                        alert("Spracheingabe konnte nicht gestartet werden.");
                                    }
                                })
                                .catch(function(err) {
                                     if(debugEnabled) console.warn("Mikrofon-Berechtigungsprüfung fehlgeschlagen, versuche trotzdem zu starten:", err);
                                     // Fallback for browsers not supporting query or if query fails
                                     try {
                                        speechRecognition.start();
                                     } catch(e) {
                                        console.error("Error starting STT (fallback):", e);
                                        alert("Spracheingabe konnte nicht gestartet werden.");
                                     }
                                });
                        } else {
                            // Fallback for browsers not supporting permissions query
                             try {
                                speechRecognition.start();
                             } catch(e) {
                                console.error("Error starting STT (no permission query):", e);
                                alert("Spracheingabe konnte nicht gestartet werden.");
                             }
                        }
                    }
                });
            } else if (settings.enable_stt && voiceButton.length) {
                // If button exists but SR failed init
                if(debugEnabled) console.warn("MC JS: STT aktiviert, aber Spracherkennung Initialisierung fehlgeschlagen.");
                voiceButton.hide();
            }
        } else if (settings.enable_stt && voiceButton.length) {
            // If STT enabled in settings but button exists and browser doesn't support STT
            voiceButton.hide();
            if(debugEnabled) console.warn("MC JS: STT aktiviert, aber vom Browser nicht unterstützt.");
        }

        // Check TTS support
        if (!synthesis && settings.enable_tts) {
            if(debugEnabled) console.warn("MC JS: TTS aktiviert, aber vom Browser nicht unterstützt.");
            // Maybe hide TTS buttons dynamically if needed, though currently added in displayMessage
        }

        // Initialize Auto Open Feature
        initializeAutoOpen();
    }


    // --- Toggle Widget (Handles MANUAL open/close) ---
    function toggleWidgetManual() {
        clearTimeout(inactivityTimer); // Stop timer on manual interaction

        if (isMinimized) { // ---- Open Manually ----
            widget.removeClass('minimized').addClass('maximized'); // Set classes for styling
            body.css('display', 'flex'); // Ensure body is flex before animating height
            if(debugEnabled) console.log("MC JS: Manual Open Start");
            widget.stop().animate({ height: maximizedHeight }, 250, function() {
                // After animation completes
                 if(debugEnabled) console.log("MC JS: Manual Open Animation End");
                 inputField.focus();
                 // Show initial greeting only if NO other messages exist except potential proactive ones
                 if (messagesContainer.find('.message-wrapper:not(.initial-greeting):not(.proactive-greeting)').length === 0) {
                     // Ensure initial greeting isn't already there
                     if(messagesContainer.find('.initial-greeting').length === 0) {
                        if(settings.initial_greeting) {
                           if(debugEnabled) console.log("MC JS: Manually opened empty chat, showing initial greeting.");
                           const greetingElement = displayMessage(settings.initial_greeting, 'bot', null, false, null); // Pass null for ajaxResponse here
                           if(greetingElement) greetingElement.addClass('initial-greeting');
                        }
                     }
                 }
                 scrollToBottom();
                 isMinimized = false; // Update state *after* opening is complete
            });
        } else { // ---- Close Manually ----
            hideOptionsPopup();
            stopTTS(); // Stop any ongoing speech synthesis
             if(debugEnabled) console.log("MC JS: Manual Close Start");
            // Animate height of the widget itself back to minimized state height
            // Hide body content *before* animating height to prevent weird rendering
            body.hide();
            widget.stop().animate({ height: minimizedHeight }, 250, function() {
                 // After animation completes
                 if(debugEnabled) console.log("MC JS: Manual Close Animation End");
                 widget.removeClass('maximized').addClass('minimized'); // Ensure classes are correct
                 isMinimized = true; // Update state *after* closing is complete
                 // Restart timer if auto-open enabled
                 if (settings.enable_auto_open) {
                    startInactivityTimer();
                 }
            });
        }
    }
    // Event listeners for manual toggle
    // Only toggle on header click if not clicking controls
    header.on('click', function(e) {
        if ($(e.target).closest('.minichat-header-controls').length === 0) {
            toggleWidgetManual();
        }
    });
    minimizeBtn.on('click', function(e) {
        e.stopPropagation(); // Prevent header click from firing too
        if (!isMinimized) {
            toggleWidgetManual();
        }
    });


    // --- Display Message Function ---
    /**
     * Displays a message in the chat window.
     * @param {string} text - The message text.
     * @param {string} sender - 'user' or 'bot'.
     * @param {number|null} messageId - The database ID of the message (for bot messages with feedback).
     * @param {boolean} isLoading - If true, display thinking indicator.
     * @param {object|null} ajaxResponseData - The full 'data' object from the AJAX response (for bot messages).
     * @returns {jQuery|null} The created message wrapper element or null.
     */
    function displayMessage(text, sender, messageId = null, isLoading = false, ajaxResponseData = null) {
        // Basic validation
        if (typeof text !== 'string' || (text.trim() === '' && !isLoading)) {
            // Allow empty text only if it's the loading indicator
            return null;
        }
         if (sender !== 'user' && sender !== 'bot') {
             return null;
         }

        // --- Create Elements ---
        const messageWrapper = $('<div>').addClass('message-wrapper ' + sender + '-message-wrapper');
        const messageElement = $('<div>').addClass('message ' + sender + '-message');
        const avatarElement = $('<span>').addClass('minichat-avatar ' + sender);
        const messageContent = $('<div>').addClass('message-content'); // Wrapper for main text

        // Set Avatar Image (if URL provided in settings)
        const avatarUrl = (sender === 'bot') ? settings.avatar_bot_url : settings.avatar_user_url; // User avatar could also be a setting
        if (avatarUrl) {
            // Basic validation for URL format might be good here
            avatarElement.css('background-image', 'url("' + escapeAttribute(avatarUrl) + '")').css('background-size', 'cover');
        }
        // Note: CSS handles the default Dashicon fallback if no background-image is set.

        // Handle Loading Indicator ("Thinking Dots")
        if (isLoading) {
            messageElement.addClass('thinking');
            // Add a placeholder span for the middle dot animation target
            messageContent.append('<span class="loading-dot-placeholder"></span>');
        } else {
            // Set message text securely
            messageContent.text(text); // Use .text() to prevent XSS from message content
        }

        // Append main content to message bubble
        messageElement.append(messageContent);

        // Add TTS Button (if enabled, bot message, not loading, and supported)
        if (sender === 'bot' && settings.enable_tts && synthesis && !isLoading && text.trim() !== '') {
            const ttsButton = $('<button>')
                .addClass('minichat-tts-button')
                .attr({'title': 'Vorlesen', 'aria-label': 'Vorlesen'})
                // .data('text', text) // Storing large text in data might be inefficient, get from element later
                .html('<span class="dashicons dashicons-controls-volumeon"></span>');
            // Append button after the content, maybe within its own container if layout needs it
            messageElement.append(ttsButton);
        }

        // Add Feedback Controls (if enabled, bot message, has ID, not loading)
        if (sender === 'bot' && messageId && settings.enable_feedback && !isLoading) {
            const feedbackContainer = $('<div>').addClass('minichat-feedback-container').attr('data-message-id', messageId);
            const thumbsUpButton = $('<button>').addClass('minichat-feedback-button positive').attr('title', 'Hilfreich').html('<span class="dashicons dashicons-thumbs-up"></span>');
            const thumbsDownButton = $('<button>').addClass('minichat-feedback-button negative').attr('title', 'Nicht hilfreich').html('<span class="dashicons dashicons-thumbs-down"></span>');

            // Feedback form (initially hidden by CSS)
            const feedbackForm = $('<div>').addClass('minichat-feedback-form');
            const feedbackTextarea = $('<textarea>').attr('placeholder', 'Optional: Begründung...');
            const feedbackSubmitButton = $('<button>').addClass('submit-feedback').text('Senden');
            const feedbackCancelButton = $('<button>').addClass('cancel-feedback').text('Abbrechen');
            feedbackForm.append(feedbackTextarea, feedbackSubmitButton, feedbackCancelButton);

            feedbackContainer.append(thumbsUpButton, thumbsDownButton);
            messageElement.append(feedbackContainer, feedbackForm); // Append controls and form to bubble
        }

        // Assemble final message structure (User: Bubble then Avatar, Bot: Avatar then Bubble)
        if (sender === 'user') {
            messageWrapper.append(messageElement).append(avatarElement);
        } else {
            messageWrapper.append(avatarElement).append(messageElement);
        }

        // Append to messages container
        messagesContainer.append(messageWrapper);

        // Scroll to bottom (only if not loading, otherwise scroll happens after content arrives)
        if (!isLoading) {
            scrollToBottom();
        }

        // --- WC ADDON EVENT START ---
        // Trigger event *after* the basic message is added to the DOM
        // Pass relevant data for potential add-ons
        $(document).trigger('minichat:after_display_message', {
            messageElement: messageWrapper,   // The jQuery object for the entire <div class="message-wrapper...">
            sender: sender,                   // 'user' or 'bot'
            messageText: text,                // The raw text displayed
            messageId: messageId,             // The DB ID (if available)
            isLoading: isLoading,             // If it was a thinking indicator
            ajaxResponseData: ajaxResponseData // The full 'data' part of the AJAX response (null for user messages or if error)
        });
        // --- WC ADDON EVENT END ---

        return messageWrapper; // Return the created element
    }


    // --- Simple HTML Escaping Helper ---
    function escapeHtml(text) {
        if (text === null || typeof text === 'undefined') return '';
        const map = { '&': '&amp;', '<': '&lt;', '>': '&gt;', '"': '&quot;', "'": '&#039;' };
        return String(text).replace(/[&<>"']/g, function(m) { return map[m]; });
    }
    // Separate escape for attributes
     function escapeAttribute(text) {
         if (text === null || typeof text === 'undefined') return '';
         const map = { '"': '&quot;', "'": '&#039;', '<': '&lt;', '>': '&gt;' };
         return String(text).replace(/["'<>]/g, function(m) { return map[m]; });
     }

    // --- Scroll to Bottom ---
    function scrollToBottom() {
        messagesContainer.stop().animate({scrollTop: messagesContainer[0].scrollHeight}, 300);
    }

    // --- Send Message Function ---
    function sendMessage() {
        const messageText = inputField.val().trim();
        if (messageText === '') return; // Don't send empty messages

        // Stop voice input if active
        if (isRecording && speechRecognition) {
            speechRecognition.stop();
        }

        // Display user message immediately
        displayMessage(messageText, 'user', null, false, null); // No ID, not loading, no ajax response for user message

        // Clear input and disable controls
        inputField.val('');
        inputField.prop('disabled', true);
        sendButton.prop('disabled', true);
        if (voiceButton.length) voiceButton.prop('disabled', true);

        // Show thinking indicator
        const loadingIndicator = displayMessage('', 'bot', null, true, null); // Display thinking indicator

        // Stop any currently playing TTS
        stopTTS();

        // Prepare data for AJAX request
        const dataToSend = {
            action: 'minichat_send_message',
            nonce: nonces.message, // Use nonce from localized data
            message: messageText   // Send raw text, sanitization happens server-side
        };

        // Perform AJAX request
        $.ajax({
            url: ajax_url, // Use URL from localized data
            type: 'POST',
            data: dataToSend,
            dataType: 'json',
            success: function(response) {
                // Remove thinking indicator regardless of success/error of the content
                if (loadingIndicator) loadingIndicator.remove();

                if (response.success && response.data && response.data.reply) {
                    // --- WC ADDON EVENT CHANGE START ---
                    // Display bot reply and pass the full response.data for the trigger
                    displayMessage(response.data.reply, 'bot', response.data.message_id, false, response.data);
                    // --- WC ADDON EVENT CHANGE END ---

                    // Handle new user ID if provided (for cookie setting, although cookie set via PHP usually)
                     if (response.data.new_user_id) {
                         if (debugEnabled) console.log("MC JS: Received new user ID hint from backend:", response.data.new_user_id);
                         // JS cannot set HttpOnly cookies, this is just an FYI from the backend.
                     }

                } else {
                    // Handle errors reported by the backend
                    let errorMsg = 'Unbekannter Fehler.';
                    if (response.data && response.data.error) {
                        errorMsg = response.data.error;
                    } else if (response.data && typeof response.data === 'string') {
                        // Sometimes PHP might echo an error string directly in data on failure
                        errorMsg = response.data;
                    }
                    console.error('Minimal Chatbot - Backend Error:', response);
                    displayMessage('Fehler: ' + escapeHtml(errorMsg), 'bot', null, false, response.data).addClass('error'); // Pass response data even on error
                }
            },
            error: function(jqXHR, textStatus, errorThrown) {
                // Handle AJAX communication errors
                if (loadingIndicator) loadingIndicator.remove();
                console.error('Minimal Chatbot - AJAX Error:', textStatus, errorThrown, jqXHR.responseText);
                displayMessage('Fehler: Verbindung fehlgeschlagen (' + escapeHtml(textStatus) + ')', 'bot', null, false, null).addClass('error'); // No valid ajax response data here
            },
            complete: function() {
                // Re-enable controls after request finishes (success or error)
                inputField.prop('disabled', false);
                sendButton.prop('disabled', false);
                if (voiceButton.length) voiceButton.prop('disabled', false);
                // Refocus input field if widget is not minimized
                if (!isMinimized) {
                    inputField.focus();
                }
                 // Ensure scroll is at bottom after potential error message + controls re-enabled
                 scrollToBottom();
            }
        });
    }

    // --- Event Listeners ---
    sendButton.on('click', sendMessage);
    inputField.on('keypress', function(event) {
        // Send on Enter key, unless Shift+Enter is pressed (for newline)
        if (event.key === 'Enter' && !event.shiftKey) {
            event.preventDefault(); // Prevent default form submission/newline
            sendMessage();
        }
    });

    // --- TTS Button Click ---
    if (settings.enable_tts && synthesis) {
        // Use event delegation for dynamically added buttons
        messagesContainer.on('click', '.minichat-tts-button', function(e) {
            e.stopPropagation(); // Prevent message click events
            const button = $(this);
            const messageElement = button.closest('.message');
            const textToSpeak = messageElement.find('.message-content').text().trim(); // Get text from content div

            if (!textToSpeak) return; // Don't speak empty messages

            if (button.hasClass('speaking') && currentUtterance) {
                // If already speaking this message, stop it
                stopTTS();
            } else {
                // Stop any other ongoing TTS first
                stopTTS();

                // Create new utterance
                currentUtterance = new SpeechSynthesisUtterance(textToSpeak);
                try {
                    // Attempt to set language based on page language
                    currentUtterance.lang = $('html').attr('lang') || 'de-DE';
                } catch (e) {
                     if(debugEnabled) console.warn("MC JS: Could not set TTS lang", e);
                }

                // Event handlers for the utterance
                currentUtterance.onstart = function() {
                    messagesContainer.find('.minichat-tts-button').removeClass('speaking'); // Reset others
                    button.addClass('speaking'); // Mark this button as active
                };
                currentUtterance.onend = function() {
                    button.removeClass('speaking');
                    currentUtterance = null; // Clear current utterance reference
                };
                currentUtterance.onerror = function(event) {
                    console.error('TTS Synthesis Error:', event.error);
                    alert('Fehler bei der Sprachausgabe.');
                    button.removeClass('speaking');
                    currentUtterance = null;
                };

                // Speak the utterance
                synthesis.speak(currentUtterance);
            }
        });
    }
    // Function to stop TTS
    function stopTTS() {
        if (synthesis && synthesis.speaking) {
            synthesis.cancel(); // Cancel any ongoing speech
        }
        // Reset button state even if cancel was called externally
        if (currentUtterance) {
            messagesContainer.find('.minichat-tts-button.speaking').removeClass('speaking');
            currentUtterance = null; // Clear reference
        }
    }

    // --- Feedback Event Delegation ---
    // Show feedback form on button click
    messagesContainer.on('click', '.minichat-feedback-button', function() {
        const $button = $(this);
        const $container = $button.closest('.minichat-feedback-container');
        const $form = $container.siblings('.minichat-feedback-form'); // Get the form associated with these buttons
        const $textarea = $form.find('textarea');

        // If this button is already selected, maybe hide form? Or just keep it open. Let's keep open.
        // if ($button.hasClass('selected')) {
        //     $form.slideUp(200);
        //     $button.removeClass('selected');
        //     return;
        // }

        // Deselect other button in the same group, select this one
        $container.find('.minichat-feedback-button').removeClass('selected');
        $button.addClass('selected');

        // Store rating choice on the form element
        const rating = $button.hasClass('positive') ? 1 : 0;
        $form.data('rating', rating); // Store 1 for positive, 0 for negative

        // Show form and focus textarea
        if (!$form.is(':visible')) {
            $form.slideDown(200, function() {
                $textarea.focus();
            });
        } else {
             $textarea.focus(); // Focus if already visible
        }
    });
    // Cancel feedback form
    messagesContainer.on('click', '.cancel-feedback', function() {
        const $button = $(this);
        const $form = $button.closest('.minichat-feedback-form');
        const $container = $form.siblings('.minichat-feedback-container');

        $form.slideUp(200);
        $form.find('textarea').val(''); // Clear textarea
        $container.find('.minichat-feedback-button').removeClass('selected'); // Deselect thumbs
        $form.removeData('rating'); // Clear stored rating
    });
    // Submit feedback form
    messagesContainer.on('click', '.submit-feedback', function() {
        const $submitButton = $(this);
        const $form = $submitButton.closest('.minichat-feedback-form');
        const $container = $form.siblings('.minichat-feedback-container');
        const $textarea = $form.find('textarea');
        const $cancelButton = $form.find('.cancel-feedback');

        const messageId = $container.data('message-id');
        const rating = $form.data('rating'); // Get stored rating
        const feedbackText = $textarea.val().trim();

        // Validate that a rating was selected
        if (typeof rating === 'undefined' || rating === null || messageId <= 0) {
            console.error("Feedback Error: Missing message ID or rating selection.");
            alert("Bitte wählen Sie zuerst Daumen hoch oder runter."); // Please select thumbs up or down first.
            return;
        }

        // Disable form elements during submission
        $textarea.prop('disabled', true);
        $submitButton.prop('disabled', true).text('Sende...'); // Change button text
        $cancelButton.prop('disabled', true);
        $container.find('.minichat-feedback-button').addClass('disabled'); // Visually disable thumbs too


        // AJAX request to save feedback
        $.ajax({
            url: ajax_url,
            type: 'POST',
            data: {
                action: 'minichat_send_feedback',
                nonce: nonces.feedback, // Use feedback nonce
                message_id: messageId,
                rating: rating,
                feedback_text: feedbackText
            },
            dataType: 'json',
            success: function(response) {
                if (response.success) {
                    // Show thank you message, then hide form
                    $form.html('<p class="feedback-thanks" style="color: green; font-style: italic; margin: 10px 0;">Danke für dein Feedback!</p>');
                    setTimeout(function() {
                        $form.slideUp(300, function() { $(this).remove(); }); // Remove form from DOM after hiding
                        // Permanently disable buttons for this message
                        $container.find('.minichat-feedback-button')
                                  .off('click') // Remove click listener
                                  .addClass('disabled')
                                  .css('cursor', 'default');
                    }, 2000);
                } else {
                    // Show error message from backend
                    alert('Fehler beim Speichern des Feedbacks: ' + (response.data.message || 'Unbekannter Fehler'));
                    // Re-enable form on error
                    $textarea.prop('disabled', false);
                    $submitButton.prop('disabled', false).text('Senden');
                    $cancelButton.prop('disabled', false);
                    $container.find('.minichat-feedback-button').removeClass('disabled');
                }
            },
            error: function(jqXHR, textStatus, errorThrown) {
                console.error("Feedback AJAX Error:", textStatus, errorThrown);
                alert('Fehler: Feedback konnte nicht gesendet werden (' + textStatus + ')');
                // Re-enable form on AJAX error
                $textarea.prop('disabled', false);
                $submitButton.prop('disabled', false).text('Senden');
                $cancelButton.prop('disabled', false);
                $container.find('.minichat-feedback-button').removeClass('disabled');
            }
        });
    });


    // --- Options Popup Logic ---
    function showOptionsPopup() {
        optionsPopup.slideDown(150);
    }
    function hideOptionsPopup() {
        optionsPopup.slideUp(150);
    }
    settingsBtn.on('click', function(e) {
        e.stopPropagation(); // Prevent header click
        if (optionsPopup.is(':visible')) {
            hideOptionsPopup();
        } else {
            showOptionsPopup();
        }
    });
    // Close popup if clicking outside of it
    $(document).on('click', function(e) {
        if (optionsPopup.is(':visible') && !$(e.target).closest('#minichat-options-popup').length && !$(e.target).closest('.minichat-settings-btn').length) {
            hideOptionsPopup();
        }
    });

    // --- Clear Chat Button Logic ---
    clearChatBtn.on('click', function() {
        // Use a more specific confirmation message
        if (!confirm('Möchtest du den gesamten Chatverlauf auf diesem Gerät wirklich löschen? Dein Verlauf auf anderen Geräten bleibt bestehen.')) {
            return;
        }
        hideOptionsPopup();

        // --- Clear visually ---
        const initialGreetingText = settings.initial_greeting || 'Hallo! Frag mich etwas.';
        // Fade out messages, clear container, add initial greeting, fade in
        messagesContainer.stop().fadeOut(200, function() {
            $(this).empty(); // Remove all messages
            // Add back the initial greeting using displayMessage
            const greetingElement = displayMessage(initialGreetingText, 'bot', null, false, null);
             if (greetingElement) greetingElement.addClass('initial-greeting');
            $(this).fadeIn(200); // Fade container back in
            stopTTS(); // Stop TTS if it was reading something
        });

        // --- Clear on server ---
        $.ajax({
            url: ajax_url,
            type: 'POST',
            data: {
                action: 'minichat_clear_chat',
                nonce: nonces.clear // Use clear chat nonce
            },
            dataType: 'json',
            success: function(response) {
                if (!response.success) {
                    // Optionally notify user of server-side clear failure
                    console.error("Fehler beim Löschen des Server-Verlaufs:", response.data.message);
                    // alert('Fehler beim Löschen des Server-Verlaufs: ' + (response.data.message || 'Unknown error'));
                } else {
                     if(debugEnabled) console.log("MC JS: Server-side history cleared successfully.");
                }
            },
            error: function(jqXHR, textStatus, errorThrown) {
                console.error("Clear Chat AJAX Error:", textStatus, errorThrown);
                 // Optionally notify user
                 // alert('Fehler: Server-Verlauf konnte nicht gelöscht werden (' + textStatus + ')');
            }
        });
    });


    // --- Auto Open Logic ---
    function initializeAutoOpen() {
        if (!settings.enable_auto_open) {
            if(debugEnabled) console.log("MC JS: Auto-open disabled in settings.");
            return;
        }

        // Check page condition
        const currentPage = window.location.pathname; // Get current path
        const isHomepage = (currentPage === '/' || settings.is_front_page === true); // settings.is_front_page passed from PHP
        if (settings.auto_open_pages === 'homepage' && !isHomepage) {
             if(debugEnabled) console.log("MC JS: Auto-open page condition failed (Homepage only). Current page:", currentPage, "Is homepage:", isHomepage);
            return;
        }

        // Check frequency condition (Session Storage)
        if (settings.auto_open_frequency === 'once_per_session') {
            try {
                if (sessionStorage.getItem('minichat_auto_opened') === 'true') {
                    if(debugEnabled) console.log("MC JS: Auto-open frequency condition failed (Already opened this session).");
                    return;
                }
            } catch (e) {
                // Session storage might be disabled or unavailable
                console.warn("MC JS: Cannot access sessionStorage to check auto-open frequency.", e);
                // Decide how to proceed: maybe default to not opening, or allow opening anyway? Let's allow opening.
            }
        }

        if(debugEnabled) console.log("MC JS: Auto-open conditions met, initializing timer.");
        startInactivityTimer(); // Start the timer only if all conditions passed

        // Reset timer on user activity (use namespaced events for easy removal if needed)
        // Throttle/debounce might be better for performance but basic reset is ok for now
        $(document).on('mousemove.minichat keydown.minichat scroll.minichat touchstart.minichat', resetInactivityTimer);
    }

    function startInactivityTimer() {
        clearTimeout(inactivityTimer); // Clear existing timer
        const delayMs = parseInt(settings.inactivity_delay || 30, 10) * 1000; // Ensure integer, use default
        if (delayMs <= 0) { // Safety check for invalid delay
            if(debugEnabled) console.log("MC JS: Invalid inactivity delay, auto-open timer not started.");
            return;
        }
         if(debugEnabled) console.log("MC JS: Starting inactivity timer (" + (delayMs / 1000) + "s).");
        inactivityTimer = setTimeout(triggerProactiveOpen, delayMs);
    }

    // Resets the timer
    function resetInactivityTimer() {
        // Only reset if auto-open is generally enabled and timer should be running
         if (settings.enable_auto_open && isMinimized) { // Only reset if currently minimized
             startInactivityTimer();
         }
    }

    // Function called by the timer when inactivity duration is reached
    function triggerProactiveOpen() {
        if (isMinimized) { // Double-check: Only trigger if still minimized
            if(debugEnabled) console.log("MC JS: Inactivity timer expired, triggering proactive open.");

            // Set session flag BEFORE opening (if frequency is 'once_per_session')
            if (settings.auto_open_frequency === 'once_per_session') {
                 try {
                    sessionStorage.setItem('minichat_auto_opened', 'true');
                    if(debugEnabled) console.log("MC JS: Set sessionStorage 'minichat_auto_opened' flag.");
                 }
                 catch (e) {
                    console.warn("MC JS: Cannot set sessionStorage flag.", e);
                 }
            }

            // Call the dedicated function to handle the opening and message display
            displayProactiveWidget();
        } else {
             if(debugEnabled) console.log("MC JS: Inactivity timer expired, but widget was already open.");
        }
    }

    // --- Dedicated function to open proactively and show correct message ---
    function displayProactiveWidget() {
         // Should only be called if minimized, but check again
         if (!isMinimized) return;
         if (debugEnabled) console.log("MC JS: Opening proactively & showing proactive message.");

         // Stop the inactivity timer loop (important!)
         clearTimeout(inactivityTimer);
         // Might also want to unbind the activity listeners? Or rely on toggleWidgetManual to restart timer if closed again. Let's keep listeners active for now.

         // --- Perform the opening animation ---
         widget.removeClass('minimized').addClass('maximized'); // Set classes for styling
         body.css('display', 'flex').hide(); // Ensure body is display:flex but start hidden for animation

         // Animate widget height
         widget.stop().animate({ height: maximizedHeight }, 250, function() {
             body.show(); // Show body content AFTER height is animated

             // --- Display the correct message ---
             let messageToShow = settings.proactive_message || settings.initial_greeting || 'Hallo! Kann ich helfen?'; // Fallback chain
             let messageClass = settings.proactive_message ? 'proactive-greeting' : 'initial-greeting';

             // Avoid adding duplicate greeting if the *very last* message is already it.
             const lastBotMessageWrapper = messagesContainer.find('.message-wrapper.bot-message-wrapper:last');
             let alreadyShown = false;
              if (lastBotMessageWrapper.length > 0 && lastBotMessageWrapper.hasClass(messageClass)) {
                  const lastBotText = lastBotMessageWrapper.find('.message-content').text().trim();
                  if (lastBotText === messageToShow) {
                      alreadyShown = true;
                       if (debugEnabled) console.log("MC JS: Proactive/initial greeting seems to be the last message already.");
                  }
              }

             if (!alreadyShown) {
                 if (debugEnabled) console.log("MC JS: Displaying proactive/initial message:", messageToShow);
                 const msgElement = displayMessage(messageToShow, 'bot', null, false, null); // No ID, not loading, no ajax response
                 if(msgElement) msgElement.addClass(messageClass);
             }

             // Focus input and scroll
             inputField.focus();
             scrollToBottom(); // Scroll after potential message display
         });
         isMinimized = false; // Update state *after* animation starts/completes
    }
    // --- End Auto Open Logic ---


    // --- Initial Setup on Page Load ---
    initializeFeatures(); // Initialize Mic/TTS AND Auto-Open timer logic (if conditions met)

    // Ensure initial state matches classes (especially on reload)
    if (widget.hasClass('minimized')) {
        body.hide(); // Make sure body is hidden
        isMinimized = true;
         if(debugEnabled) console.log("MC JS: Initial state is minimized.");
    } else {
        body.show(); // Make sure body is visible
        isMinimized = false;
        inputField.focus(); // Focus input if starting maximized
        scrollToBottom(); // Scroll on load if maximized
         if(debugEnabled) console.log("MC JS: Initial state is maximized.");
    }

}); // End jQuery ready